import datetime
import random

from PIL import Image, ImageDraw
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import Maker, make_gif_or_combined_gif


class Dot:
    def __init__(self, positon: tuple[int, int], direction: tuple[float, float]):
        self.px = positon[0]
        self.py = positon[1]
        self.vx = 0
        self.vy = 0
        self.dx = direction[0]
        self.dy = direction[1]
        self.radius = random.randint(1, 3)

    def move(self, step: int):
        a = 0.02 * step / self.radius
        self.vx += a * self.dx
        self.vy += a * self.dy
        self.px += round(self.vx)
        self.py += round(self.vy)
        if random.random() < 0.25:
            self.radius -= 1

    def draw_on(self, img: Image.Image):
        ImageDraw.Draw(img).ellipse(
            (
                self.px - self.radius,
                self.py - self.radius,
                self.px + self.radius,
                self.py + self.radius,
            ),
            fill=(0, 0, 0, 255),
        )

    def out_of_img(self, img: Image.Image) -> bool:
        w, h = img.size
        if (
            self.radius <= 0
            or self.px + self.radius < 0
            or self.px - self.radius > w
            or self.py + self.radius < 0
            or self.py - self.radius > h
        ):
            return True
        return False


def make_dust(dusts: list[Dot], step: int, size: tuple[int, int]) -> Image.Image:
    dust_mask = Image.new("RGBA", size)
    remove_list = []
    for dot in dusts:
        dot.move(step)
        if dot.out_of_img(dust_mask):
            remove_list.append(dot)
        else:
            dot.draw_on(dust_mask)
    for dot in remove_list:
        dusts.remove(dot)
    return dust_mask


def fade_away(images: list[BuildImage], texts, args):
    image = images[0]
    width, height = image.size
    area = width * height
    if area >= 40000:
        width = round(200 * width / (area**0.5))
        height = round(200 * height // (area**0.5))
    o = (width * 2 // 3, height * 3 // 2)
    step = (o[0] ** 2 + o[1] ** 2) ** 0.5 / 24
    dusts = []

    def maker(i: int) -> Maker:
        def make(imgs: list[BuildImage]) -> BuildImage:
            img = imgs[0].image.convert("RGBA").resize((width, height))
            if i <= 9:
                return BuildImage(img)
            elif 9 < i < 28:
                new_img = img.copy()
                for x in range(width):
                    for y in range(
                        max(0, min(height, height - round(step * (i + 11)))), height
                    ):
                        pixel = (x, y)
                        if img.getpixel(pixel)[3] == 0:  # type: ignore
                            continue
                        distance = ((x - o[0]) ** 2 + (y - o[1]) ** 2) ** 0.5
                        if distance <= step * (i - 5):
                            new_img.putpixel(pixel, (0, 0, 0, 0))
                        elif distance <= step * (i - 4):
                            new_img.putpixel(pixel, (0, 0, 0, 255))
                            if random.random() <= 0.06:
                                direction = (
                                    (x - o[0]) / distance,
                                    (y - o[1] * 1.5) / distance,
                                )
                                dusts.append(Dot((x, y), direction))
                        elif distance <= step * (i + 2):
                            factor = (distance - step * (i - 11)) / (step * 12)
                            factor = max(0, min(1, factor))
                            factor *= 0.9 + 0.2 * random.random()
                            value = img.getpixel(pixel)
                            gray = (value[0] + value[1] + value[2]) / 3  # type: ignore
                            gray = round(gray * factor)
                            new_color = (gray, gray, gray, value[3])  # type: ignore
                            new_img.putpixel(pixel, new_color)
                        else:
                            continue
                dust_mask = make_dust(dusts, step, img.size)
                new_img.paste(dust_mask, (0, 0), dust_mask)
                return BuildImage(new_img)
            else:
                return BuildImage(make_dust(dusts, step, img.size))

        return make

    return make_gif_or_combined_gif(images, maker, 35, 0.08)


add_meme(
    "fade_away",
    fade_away,
    max_images=1,
    min_images=1,
    keywords=["灰飞烟灭"],
    date_created=datetime.datetime(2024, 8, 20),
    date_modified=datetime.datetime(2024, 8, 21),
)



from datetime import datetime

from pil_utils import BuildImage

from meme_generator import add_meme


def call_110(images: list[BuildImage], texts, args):
    img1 = images[0].convert("RGBA").square().resize((250, 250))
    img0 = images[1].convert("RGBA").square().resize((250, 250))

    frame = BuildImage.new("RGB", (900, 500), "white")
    frame.draw_text((0, 0, 900, 200), "遇到困难请拨打", max_fontsize=100)
    frame.paste(img1, (50, 200), alpha=True)
    frame.paste(img1, (325, 200), alpha=True)
    frame.paste(img0, (600, 200), alpha=True)
    return frame.save_jpg()


add_meme(
    "call_110",
    call_110,
    min_images=2,
    max_images=2,
    keywords=["遇到困难请拨打"],
    date_created=datetime(2022, 8, 26),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"


def run(images, texts: list[str], args):
    text = texts[0]
    frame = BuildImage.open(img_dir / "0.png")
    text_img = BuildImage.new("RGBA", (122, 53))
    try:
        text_img.draw_text(
            (0, 0, 122, 53),
            text,
            allow_wrap=True,
            max_fontsize=50,
            min_fontsize=10,
            lines_align="center",
        )
    except ValueError:
        raise TextOverLength(text)
    frame.paste(text_img.rotate(7, expand=True), (200, 195), alpha=True)
    return frame.save_jpg()


add_meme(
    "run",
    run,
    min_texts=1,
    max_texts=1,
    default_texts=["快跑"],
    keywords=["快跑"],
    date_created=datetime(2022, 10, 17),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime

from PIL import ImageOps
from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength
from meme_generator.utils import save_gif


def flash_blind(images: list[BuildImage], texts: list[str], args):
    img = images[0].convert("RGB").resize_width(500)
    frames: list[IMG] = []
    frames.append(img.image)
    frames.append(ImageOps.invert(img.image))
    img_enlarge = img.resize_canvas((450, img.height * 450 // 500)).resize(
        (500, img.height)
    )
    frames.append(img_enlarge.image)
    frames.append(ImageOps.invert(img.image))

    if texts and texts[0]:
        text = texts[0]
        text_h = 65

        try:
            text_frame_black = BuildImage.new("RGB", (500, text_h), "black")
            text_frame_white = BuildImage.new("RGB", (500, text_h), "white")
            text_frame_black.draw_text(
                (10, 0, 490, text_h),
                text,
                max_fontsize=50,
                min_fontsize=20,
                fill="white",
            )
            text_frame_white.draw_text(
                (10, 0, 490, text_h),
                text,
                max_fontsize=50,
                min_fontsize=20,
                fill="black",
            )
        except ValueError:
            raise TextOverLength(text)
        frames[0].paste(text_frame_black.image, (0, img.height - text_h))
        frames[1].paste(text_frame_white.image, (0, img.height - text_h))
        frames[2].paste(text_frame_black.image, (0, img.height - text_h))
        frames[3].paste(text_frame_white.image, (0, img.height - text_h))

    return save_gif(frames, 0.03)


add_meme(
    "flash_blind",
    flash_blind,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    default_texts=["闪瞎你们的狗眼"],
    keywords=["闪瞎"],
    date_created=datetime(2023, 5, 5),
    date_modified=datetime(2023, 5, 5),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"


def ascension(images, texts: list[str], args):
    frame = BuildImage.open(img_dir / "0.png")
    text = f"你原本应该要去地狱的，但因为你生前{texts[0]}，我们就当作你已经服完刑期了"
    try:
        frame.draw_text(
            (40, 30, 482, 135),
            text,
            allow_wrap=True,
            max_fontsize=50,
            min_fontsize=20,
        )
    except ValueError:
        raise TextOverLength(texts[0])
    return frame.save_jpg()


add_meme(
    "ascension",
    ascension,
    min_texts=1,
    max_texts=1,
    default_texts=["学的是机械"],
    keywords=["升天"],
    date_created=datetime(2022, 10, 17),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def kiss(images: list[BuildImage], texts, args):
    self_head = images[0].convert("RGBA").circle().resize((40, 40))
    user_head = images[1].convert("RGBA").circle().resize((50, 50))
    # fmt: off
    user_locs = [
        (58, 90), (62, 95), (42, 100), (50, 100), (56, 100), (18, 120), (28, 110),
        (54, 100), (46, 100), (60, 100), (35, 115), (20, 120), (40, 96)
    ]
    self_locs = [
        (92, 64), (135, 40), (84, 105), (80, 110), (155, 82), (60, 96), (50, 80),
        (98, 55), (35, 65), (38, 100), (70, 80), (84, 65), (75, 65)
    ]
    # fmt: on
    frames: list[IMG] = []
    for i in range(13):
        frame = BuildImage.open(img_dir / f"{i}.png")
        frame.paste(user_head, user_locs[i], alpha=True)
        frame.paste(self_head, self_locs[i], alpha=True)
        frames.append(frame.image)
    return save_gif(frames, 0.05)


add_meme(
    "kiss",
    kiss,
    min_images=2,
    max_images=2,
    keywords=["亲", "亲亲"],
    date_created=datetime(2021, 6, 11),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def step_on(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").resize((100, 100), keep_ratio=True)
    frames: list[IMG] = []
    locs = [
        (104, 72, 32, 185, -25),
        (104, 72, 32, 185, -25),
        (90, 73, 51, 207, 0),
        (88, 78, 51, 202, 0),
        (88, 86, 49, 197, 0),
    ]
    for i in range(5):
        frame = BuildImage.open(img_dir / f"{i}.png")
        w, h, x, y, angle = locs[i]
        frame.paste(img.resize((w, h)).rotate(angle, expand=True), (x, y), below=True)
        frames.append(frame.image)
    return save_gif(frames, 0.07)


add_meme(
    "step_on",
    step_on,
    min_images=1,
    max_images=1,
    keywords=["踩"],
    date_created=datetime(2023, 3, 28),
    date_modified=datetime(2023, 3, 28),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def jiujiu(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").resize((75, 51), keep_ratio=True)
    frames: list[IMG] = []
    for i in range(8):
        frame = BuildImage.open(img_dir / f"{i}.png")
        frame.paste(img, below=True)
        frames.append(frame.image)
    return save_gif(frames, 0.06)


add_meme(
    "jiujiu",
    jiujiu,
    min_images=1,
    max_images=1,
    keywords=["啾啾"],
    date_created=datetime(2022, 4, 20),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage, Text2Image

from meme_generator import add_meme

img_dir = Path(__file__).parent / "images"


def nokia(images, texts: list[str], args):
    text = texts[0][:900]
    text_img = (
        Text2Image.from_text(text, 70, fontname="FZXS14", fill="black", spacing=30)
        .wrap(700)
        .to_image()
    )
    text_img = (
        BuildImage(text_img)
        .resize_canvas((700, 450), direction="northwest")
        .rotate(-9.3, expand=True)
    )

    head_img = Text2Image.from_text(
        f"{len(text)}/900", 70, fontname="FZXS14", fill=(129, 212, 250, 255)
    ).to_image()
    head_img = BuildImage(head_img).rotate(-9.3, expand=True)

    frame = BuildImage.open(img_dir / "0.jpg")
    frame.paste(text_img, (205, 330), alpha=True)
    frame.paste(head_img, (790, 320), alpha=True)
    return frame.save_jpg()


add_meme(
    "nokia",
    nokia,
    min_texts=1,
    max_texts=1,
    default_texts=["无内鬼，继续交易"],
    keywords=["诺基亚", "有内鬼"],
    date_created=datetime(2021, 12, 15),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import make_jpg_or_gif


def alike(images: list[BuildImage], texts, args):
    frame = BuildImage.new("RGBA", (470, 180), "white")
    frame.draw_text(
        (10, 10, 185, 140), "你怎么跟", max_fontsize=40, min_fontsize=30, halign="right"
    ).draw_text(
        (365, 10, 460, 140), "一样", max_fontsize=40, min_fontsize=30, halign="left"
    )

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA").resize((150, 150), keep_ratio=True)
        return frame.copy().paste(img, (200, 15), alpha=True)

    return make_jpg_or_gif(images, make)


add_meme(
    "alike",
    alike,
    min_images=1,
    max_images=1,
    keywords=["一样"],
    date_created=datetime(2022, 1, 2),
    date_modified=datetime(2023, 2, 22),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength
from meme_generator.tags import MemeTags

img_dir = Path(__file__).parent / "images"


def nekoha_holdsign(images, texts: list[str], args):
    text = texts[0]
    frame = BuildImage.open(img_dir / "0.jpg")
    try:
        frame.draw_text(
            (210, 520, 570, 765),
            text,
            fill=(72, 110, 173),
            allow_wrap=True,
            fontname="FZShaoEr-M11S",
            max_fontsize=65,
            min_fontsize=25,
            lines_align="center",
        )
    except ValueError:
        raise TextOverLength(text)
    return frame.save_jpg()


add_meme(
    "nekoha_holdsign",
    nekoha_holdsign,
    min_texts=1,
    max_texts=1,
    default_texts=["V我50"],
    keywords=["猫羽雫举牌", "猫猫举牌"],
    tags=MemeTags.nekoha,
    date_created=datetime(2023, 3, 30),
    date_modified=datetime(2023, 3, 30),
)



from datetime import datetime
from pathlib import Path

from arclet.alconna import store_true
from PIL.Image import Image as IMG
from pil_utils import BuildImage
from pydantic import Field

from meme_generator import MemeArgsModel, MemeArgsType, ParserOption, add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"

help_text = "是否将图片变为圆形"


class Model(MemeArgsModel):
    circle: bool = Field(False, description=help_text)


args_type = MemeArgsType(
    args_model=Model,
    args_examples=[Model(circle=False), Model(circle=True)],
    parser_options=[
        ParserOption(
            names=["--circle", "圆"],
            default=False,
            action=store_true,
            help_text=help_text,
        ),
    ],
)


def petpet(images: list[BuildImage], texts, args: Model):
    img = images[0].convert("RGBA").square()
    if args.circle:
        img = img.circle()

    frames: list[IMG] = []
    locs = [
        (14, 20, 98, 98),
        (12, 33, 101, 85),
        (8, 40, 110, 76),
        (10, 33, 102, 84),
        (12, 20, 98, 98),
    ]
    for i in range(5):
        hand = BuildImage.open(img_dir / f"{i}.png")
        frame = BuildImage.new("RGBA", hand.size, (255, 255, 255, 0))
        x, y, w, h = locs[i]
        frame.paste(img.resize((w, h)), (x, y), alpha=True)
        frame.paste(hand, alpha=True)
        frames.append(frame.image)
    return save_gif(frames, 0.06)


add_meme(
    "petpet",
    petpet,
    min_images=1,
    max_images=1,
    args_type=args_type,
    keywords=["摸", "摸摸", "摸头", "rua"],
    date_created=datetime(2021, 5, 4),
    date_modified=datetime(2023, 2, 11),
)



import json
from datetime import datetime
from typing import Any, Literal, Optional

import filetype
from fastapi import Depends, FastAPI, Form, HTTPException, Response, UploadFile
from pydantic import BaseModel, ValidationError

from meme_generator.compat import model_dump, model_json_schema, type_validate_python
from meme_generator.config import meme_config
from meme_generator.exception import (
    ArgModelMismatch,
    MemeGeneratorException,
    NoSuchMeme,
)
from meme_generator.log import LOGGING_CONFIG, setup_logger
from meme_generator.manager import get_meme, get_meme_keys, get_memes
from meme_generator.meme import CommandShortcut, Meme, MemeArgsModel, ParserOption
from meme_generator.utils import MemeProperties, render_meme_list, run_sync

app = FastAPI()


class MemeArgsResponse(BaseModel):
    args_model: dict[str, Any]
    args_examples: list[dict[str, Any]]
    parser_options: list[ParserOption]


class MemeParamsResponse(BaseModel):
    min_images: int
    max_images: int
    min_texts: int
    max_texts: int
    default_texts: list[str]
    args_type: Optional[MemeArgsResponse] = None


class MemeInfoResponse(BaseModel):
    key: str
    params_type: MemeParamsResponse
    keywords: list[str]
    shortcuts: list[CommandShortcut]
    tags: set[str]
    date_created: datetime
    date_modified: datetime


def register_router(meme: Meme):
    if args_type := meme.params_type.args_type:
        args_model = args_type.args_model
    else:
        args_model = MemeArgsModel

    def args_checker(
        args: Optional[str] = Form(default=json.dumps(model_dump(args_model()))),
    ):
        if not args:
            return MemeArgsModel()
        try:
            model = type_validate_python(args_model, json.loads(args))
        except ValidationError as e:
            e = ArgModelMismatch(str(e))
            raise HTTPException(status_code=552, detail=e.message)
        return model

    @app.post(f"/memes/{meme.key}/")
    async def _(
        images: list[UploadFile] = [],
        texts: list[str] = meme.params_type.default_texts,
        args: args_model = Depends(args_checker),  # type: ignore
    ):
        imgs: list[bytes] = []
        for image in images:
            imgs.append(await image.read())

        texts = [text for text in texts if text]

        assert isinstance(args, args_model)

        try:
            result = await run_sync(meme)(
                images=imgs, texts=texts, args=model_dump(args)
            )
        except MemeGeneratorException as e:
            raise HTTPException(status_code=e.status_code, detail=e.message)

        content = result.getvalue()
        media_type = str(filetype.guess_mime(content)) or "text/plain"
        return Response(content=content, media_type=media_type)


class MemeKeyWithProperties(BaseModel):
    meme_key: str
    disabled: bool = False
    labels: list[Literal["new", "hot"]] = []


default_meme_list = [
    MemeKeyWithProperties(meme_key=meme.key)
    for meme in sorted(get_memes(), key=lambda meme: meme.key)
]


class RenderMemeListRequest(BaseModel):
    meme_list: list[MemeKeyWithProperties] = default_meme_list
    text_template: str = "{keywords}"
    add_category_icon: bool = True


def register_routers():
    @app.post("/memes/render_list")
    def _(params: RenderMemeListRequest = RenderMemeListRequest()):
        try:
            meme_list = [
                (
                    get_meme(p.meme_key),
                    MemeProperties(disabled=p.disabled, labels=p.labels),
                )
                for p in params.meme_list
            ]
        except NoSuchMeme as e:
            raise HTTPException(status_code=e.status_code, detail=e.message)

        result = render_meme_list(
            meme_list,
            text_template=params.text_template,
            add_category_icon=params.add_category_icon,
        )
        content = result.getvalue()
        media_type = str(filetype.guess_mime(content)) or "text/plain"
        return Response(content=content, media_type=media_type)

    @app.get("/memes/keys")
    def _():
        return get_meme_keys()

    @app.get("/memes/{key}/info")
    def _(key: str):
        try:
            meme = get_meme(key)
        except NoSuchMeme as e:
            raise HTTPException(status_code=e.status_code, detail=e.message)

        args_type_response = None
        if args_type := meme.params_type.args_type:
            args_model = args_type.args_model
            args_type_response = MemeArgsResponse(
                args_model=model_json_schema(args_model),
                args_examples=[
                    model_dump(example) for example in args_type.args_examples
                ],
                parser_options=args_type.parser_options,
            )

        return MemeInfoResponse(
            key=meme.key,
            params_type=MemeParamsResponse(
                min_images=meme.params_type.min_images,
                max_images=meme.params_type.max_images,
                min_texts=meme.params_type.min_texts,
                max_texts=meme.params_type.max_texts,
                default_texts=meme.params_type.default_texts,
                args_type=args_type_response,
            ),
            keywords=meme.keywords,
            shortcuts=meme.shortcuts,
            tags=meme.tags,
            date_created=meme.date_created,
            date_modified=meme.date_modified,
        )

    @app.get("/memes/{key}/preview")
    async def _(key: str):
        try:
            meme = get_meme(key)
            result = await run_sync(meme.generate_preview)()
        except MemeGeneratorException as e:
            raise HTTPException(status_code=e.status_code, detail=e.message)

        content = result.getvalue()
        media_type = str(filetype.guess_mime(content)) or "text/plain"
        return Response(content=content, media_type=media_type)

    for meme in sorted(get_memes(), key=lambda meme: meme.key):
        register_router(meme)


def run_server():
    import uvicorn

    register_routers()
    uvicorn.run(
        app,
        host=meme_config.server.host,
        port=meme_config.server.port,
        log_config=LOGGING_CONFIG,
    )


if __name__ == "__main__":
    setup_logger()
    run_server()



from datetime import datetime

from pil_utils import BuildImage
from pydantic import Field

from meme_generator import (
    MemeArgsModel,
    MemeArgsType,
    ParserArg,
    ParserOption,
    add_meme,
)
from meme_generator.exception import MemeFeedback
from meme_generator.utils import make_jpg_or_gif

help_text = "像素化大小，默认为 10"


class Model(MemeArgsModel):
    number: int = Field(10, description=help_text)


args_type = MemeArgsType(
    args_model=Model,
    parser_options=[
        ParserOption(
            names=["-n", "--number"],
            args=[ParserArg(name="number", value="int")],
            help_text=help_text,
        ),
    ],
)


def pixelate(images: list[BuildImage], texts, args: Model):
    num = args.number
    if not (1 <= num < min(images[0].size)):
        raise MemeFeedback("请输入合适的像素化大小")

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0]
        image = img.image
        image = image.resize((img.width // num, img.height // num), resample=0)
        image = image.resize(img.size, resample=0)
        return BuildImage(image)

    return make_jpg_or_gif(images, make)


add_meme(
    "pixelate",
    pixelate,
    min_images=1,
    max_images=1,
    keywords=["像素化"],
    args_type=args_type,
    date_created=datetime(2024, 8, 12),
    date_modified=datetime(2024, 8, 12),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength
from meme_generator.utils import make_jpg_or_gif

img_dir = Path(__file__).parent / "images"

default_text = "偷学群友数理基础"


def learn(images: list[BuildImage], texts: list[str], args):
    text = texts[0] if texts else default_text
    frame = BuildImage.open(img_dir / "0.png")
    try:
        frame.draw_text(
            (100, 1360, frame.width - 100, 1730),
            text,
            max_fontsize=350,
            min_fontsize=200,
            weight="bold",
        )
    except ValueError:
        raise TextOverLength(text)

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA").resize((1751, 1347), keep_ratio=True)
        return frame.copy().paste(img, (1440, 0), alpha=True)

    return make_jpg_or_gif(images, make)


add_meme(
    "learn",
    learn,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    default_texts=[default_text],
    keywords=["偷学"],
    date_created=datetime(2022, 12, 4),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def hug_leg(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").square()
    locs = [
        (50, 73, 68, 92),
        (58, 60, 62, 95),
        (65, 10, 67, 118),
        (61, 20, 77, 97),
        (55, 44, 65, 106),
        (66, 85, 60, 98),
    ]
    frames: list[IMG] = []
    for i in range(6):
        frame = BuildImage.open(img_dir / f"{i}.png")
        x, y, w, h = locs[i]
        frame.paste(img.resize((w, h)), (x, y), below=True)
        frames.append(frame.image)
    return save_gif(frames, 0.06)


add_meme(
    "hug_leg",
    hug_leg,
    min_images=1,
    max_images=1,
    keywords=["抱大腿"],
    date_created=datetime(2022, 10, 1),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path
from typing import Literal

from arclet.alconna import store_value
from pil_utils import BuildImage
from pydantic import Field

from meme_generator import (
    MemeArgsModel,
    MemeArgsType,
    ParserArg,
    ParserOption,
    add_meme,
)
from meme_generator.utils import make_png_or_gif

help_text = "小丑在前/后，front/behind"


class Model(MemeArgsModel):
    mode: Literal["front", "behind"] = Field("front", description=help_text)


args_type = MemeArgsType(
    args_model=Model,
    args_examples=[Model(mode="front"), Model(mode="behind")],
    parser_options=[
        ParserOption(
            names=["--mode"],
            args=[ParserArg(name="mode", value="str")],
            help_text=help_text,
        ),
        ParserOption(
            names=["--front", "前"],
            dest="mode",
            action=store_value("front"),
            help_text="小丑在前",
        ),
        ParserOption(
            names=["--behind", "后"],
            dest="mode",
            action=store_value("behind"),
            help_text="小丑在后",
        ),
    ],
)


img_dir = Path(__file__).parent / "images"


def clown_mask(images: list[BuildImage], texts: list[str], args: Model):
    def make_front(imgs: list[BuildImage]) -> BuildImage:
        frame = BuildImage.open(img_dir / "0.png")
        img = imgs[0].convert("RGBA").circle().resize((440, 440)).rotate(15)
        return frame.copy().paste(img, (16, 104), below=True)

    def make_behind(imgs: list[BuildImage]) -> BuildImage:
        frame1 = BuildImage.open(img_dir / "1.png")
        frame2 = BuildImage.open(img_dir / "2.png")
        img = (
            imgs[0]
            .convert("RGBA")
            .circle()
            .perspective(((282, 0), (496, 154), (214, 546), (0, 392)))
            .rotate(6)
        )
        frame1.paste(img, (214, 100), alpha=True)
        return frame1.paste(frame2, (-85, 20), alpha=True)

    if args.mode == "front":
        return make_png_or_gif(images, make_front)
    return make_png_or_gif(images, make_behind)


add_meme(
    "clown_mask",
    clown_mask,
    min_images=1,
    max_images=1,
    keywords=["小丑面具"],
    args_type=args_type,
    date_created=datetime(2024, 9, 20),
    date_modified=datetime(2024, 9, 20),
)



import asyncio
from pathlib import Path
from typing import Any

import filetype
from arclet.alconna import (
    Alconna,
    Args,
    CommandMeta,
    MultiVar,
    Option,
    Subcommand,
    TextFormatter,
)
from arclet.alconna.exceptions import SpecialOptionTriggered
from arclet.alconna.tools import RichConsoleFormatter

from meme_generator.app import run_server
from meme_generator.config import meme_config
from meme_generator.download import check_resources
from meme_generator.exception import MemeGeneratorException, NoSuchMeme
from meme_generator.log import setup_logger
from meme_generator.manager import get_meme, get_memes


def construct_parser() -> Alconna:
    sub_commands: list[Subcommand] = []
    for meme in get_memes():
        options: list[Option] = []
        if args_type := meme.params_type.args_type:
            for option in args_type.parser_options:
                options.append(option.option())
        sub_command = Subcommand(
            meme.key,
            *options,
            Option(
                "--images",
                Args["images", MultiVar(str, "+")],
                help_text="输入图片路径",
            ),
            Option("--texts", Args["texts", MultiVar(str, "+")], help_text="输入文字"),
            help_text="/".join(meme.keywords),
        )
        sub_commands.append(sub_command)

    parser = Alconna(
        "meme",
        Subcommand("list", alias=["ls"], help_text="查看表情列表"),
        Subcommand(
            "info", Args["key#表情名", str], alias=["show"], help_text="查看表情详情"
        ),
        Subcommand("preview", Args["key#表情名", str], help_text="生成表情预览"),
        Subcommand("generate", *sub_commands, alias=["make"], help_text="制作表情"),
        Subcommand("run", alias=["start"], help_text="启动 web server"),
        Subcommand(
            "download",
            Option("--url", Args["url", str], help_text="指定资源链接"),
            help_text="下载内置表情图片",
        ),
        meta=CommandMeta(
            description="表情包生成器",
            example="meme generate petpet --images /path/to/image/file",
        ),
        formatter_type=RichConsoleFormatter,
    )
    return parser


def list_memes() -> str:
    memes = sorted(get_memes(), key=lambda meme: meme.key)
    return "\n".join(
        f"{i}. {meme.key} ({'/'.join(meme.keywords)})"
        for i, meme in enumerate(memes, start=1)
    )


def meme_info(key: str) -> str:
    try:
        meme = get_meme(key)
    except NoSuchMeme:
        return f'表情 "{key}" 不存在！'

    keywords = "、".join([f'"{keyword}"' for keyword in meme.keywords])
    shortcuts = "、".join(
        [f'"{shortcut.humanized or shortcut.key}"' for shortcut in meme.shortcuts]
    )
    tags = "、".join([f'"{tag}"' for tag in sorted(meme.tags)])

    image_num = f"{meme.params_type.min_images}"
    if meme.params_type.max_images > meme.params_type.min_images:
        image_num += f" ~ {meme.params_type.max_images}"

    text_num = f"{meme.params_type.min_texts}"
    if meme.params_type.max_texts > meme.params_type.min_texts:
        text_num += f" ~ {meme.params_type.max_texts}"

    default_texts = ", ".join([f'"{text}"' for text in meme.params_type.default_texts])

    args_info = ""
    if args_type := meme.params_type.args_type:
        formater = TextFormatter()
        for option in args_type.parser_options:
            opt = option.option()
            alias_text = (
                " ".join(opt.requires)
                + (" " if opt.requires else "")
                + "│".join(sorted(opt.aliases, key=len))
            )
            args_info += (
                f"\n  * {alias_text}{opt.separators[0]}"
                f"{formater.parameters(opt.args)} {opt.help_text}"
            )

    return (
        f"表情名：{meme.key}\n"
        + f"关键词：{keywords}\n"
        + (f"快捷指令：{shortcuts}\n" if shortcuts else "")
        + (f"标签：{tags}\n" if tags else "")
        + f"需要图片数目：{image_num}\n"
        + f"需要文字数目：{text_num}\n"
        + (f"默认文字：[{default_texts}]\n" if default_texts else "")
        + (f"其他参数：{args_info}" if args_info else "")
    )


def generate_meme_preview(key: str) -> str:
    try:
        meme = get_meme(key)
    except NoSuchMeme:
        return f'表情 "{key}" 不存在！'

    try:
        result = meme.generate_preview()
        content = result.getvalue()
        ext = filetype.guess_extension(content)
        filename = f"result.{ext}"
        with open(filename, "wb") as f:
            f.write(content)
        return f'表情制作成功！生成的表情文件为 "{filename}"'
    except MemeGeneratorException as e:
        return str(e)


def generate_meme(
    key: str, images: list[str], texts: list[str], args: dict[str, Any]
) -> str:
    try:
        meme = get_meme(key)
    except NoSuchMeme:
        return f'表情 "{key}" 不存在！'

    for image in images:
        if not Path(image).exists():
            return f'图片路径 "{image}" 不存在！'

    try:
        result = meme(images=images, texts=texts, args=args)
        content = result.getvalue()
        ext = filetype.guess_extension(content)
        filename = f"result.{ext}"
        with open(filename, "wb") as f:
            f.write(content)
        return f'表情制作成功！生成的表情文件为 "{filename}"'
    except MemeGeneratorException as e:
        return str(e)


def main():
    setup_logger()
    parser = construct_parser()
    result = parser()

    if not result.matched:
        if not isinstance(result.error_info, SpecialOptionTriggered):
            print(result.error_info)  # noqa: T201
        return

    if not result.subcommands:
        print(parser.get_help())  # noqa: T201

    for subcommand, sub_result in result.subcommands.items():
        if subcommand == "list":
            print(list_memes())  # noqa: T201

        elif subcommand == "info":
            key = str(sub_result.args["key"])
            print(meme_info(key))  # noqa: T201

        elif subcommand == "preview":
            key = str(sub_result.args["key"])
            print(generate_meme_preview(key))  # noqa: T201

        elif subcommand == "generate":
            for key, subsub_result in sub_result.subcommands.items():
                images = (
                    list(subsub_result.options["images"].args["images"])
                    if "images" in subsub_result.options
                    else []
                )
                texts = (
                    list(subsub_result.options["texts"].args["texts"])
                    if "texts" in subsub_result.options
                    else []
                )
                options = subsub_result.options
                options.pop("images", None)
                options.pop("texts", None)
                args = {}
                for option, option_result in options.items():
                    if option_result.value is None:
                        args.update(option_result.args)
                    else:
                        args[option] = option_result.value
                print(generate_meme(key, images, texts, args))  # noqa: T201

        elif subcommand == "run":
            run_server()

        elif subcommand == "download":
            if "url" in sub_result.options:
                url = sub_result.options["url"].args["url"]
                meme_config.resource.resource_url = url
            loop = asyncio.new_event_loop()
            loop.run_until_complete(check_resources())


if __name__ == "__main__":
    main()



import random
from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import Maker, make_gif_or_combined_gif

img_dir = Path(__file__).parent / "images"


def flush(images: list[BuildImage], texts, args):
    def maker(i: int) -> Maker:
        def make(imgs: list[BuildImage]):
            img = imgs[0].convert("RGBA").square()
            w, h = img.size
            if i >= 18:
                return BuildImage.open(img_dir / f"{i-18}.png").resize(
                    (w, h), keep_ratio=True
                )

            j = 0.2 * (2 * random.random() - 1)  # 抖动
            k = 8 * i  # 变红
            f = 0.01 * i  # 放大
            crop_box = (
                round(w * f + w * f * j),
                round(h * f + h * f * j),
                round(w * (1 - f) + w * f * j),
                round(h * (1 - f) + h * f * j),
            )
            croped_img = img.crop(crop_box)
            frame = BuildImage.new("RGBA", croped_img.size, "white")
            frame.paste(croped_img, (0, 0), alpha=True)
            red_filter = BuildImage.new("RGBA", croped_img.size, (255, 0, 0, k))
            frame.alpha_composite(red_filter)
            return frame.resize((w, h))

        return make

    return make_gif_or_combined_gif(images, maker, 30, 0.08)


add_meme(
    "flush",
    flush,
    min_images=1,
    max_images=1,
    keywords=["红温"],
    date_created=datetime(2024, 9, 3),
    date_modified=datetime(2024, 9, 3),
)



import math
from datetime import datetime
from pathlib import Path

from arclet.alconna import store_true
from pil_utils import BuildImage
from pydantic import Field

from meme_generator import MemeArgsModel, MemeArgsType, ParserOption, add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"

help_text = "是否将图片变为圆形"


class Model(MemeArgsModel):
    circle: bool = Field(False, description=help_text)


args_type = MemeArgsType(
    args_model=Model,
    args_examples=[Model(circle=False), Model(circle=True)],
    parser_options=[
        ParserOption(
            names=["--circle", "圆"],
            default=False,
            action=store_true,
            help_text=help_text,
        ),
    ],
)


def jiji_king(images: list[BuildImage], texts: list[str], args: Model):
    block_num = 5
    if len(images) >= 7 or len(texts) >= 7:
        block_num = max(len(images), len(texts)) - 1

    chars = ["急"]
    text = "我是急急国王"

    if len(texts) == 1:
        if len(images) == 1:
            chars = [texts[0]] * block_num
            text = f"我是{texts[0]*2}国王"
        else:
            text = texts[0]
    elif len(texts) == 2:
        chars = [texts[0]] * block_num
        text = texts[1]
    elif texts:
        chars = sum(
            [[arg] * math.ceil(block_num / len(texts[:-1])) for arg in texts[:-1]], []
        )
        text = texts[-1]

    frame = BuildImage.new("RGBA", (10 + 100 * block_num, 400), "white")
    king = BuildImage.open(img_dir / "0.png")
    head = images[0].convert("RGBA").square().resize((125, 125))
    if args.circle:
        head = head.circle()
    king.paste(head, (237, 5), alpha=True)
    frame.paste(king, ((frame.width - king.width) // 2, 0))

    if len(images) > 1:
        imgs = images[1:]
        imgs = [img.convert("RGBA").square().resize((90, 90)) for img in imgs]
    else:
        imgs = []
        for char in chars:
            block = BuildImage.new("RGBA", (90, 90), "black")
            try:
                block.draw_text(
                    (0, 0, 90, 90),
                    char,
                    lines_align="center",
                    weight="bold",
                    max_fontsize=60,
                    min_fontsize=30,
                    fill="white",
                )
            except ValueError:
                raise TextOverLength(char)
            imgs.append(block)

    imgs = sum([[img] * math.ceil(block_num / len(imgs)) for img in imgs], [])
    for i in range(block_num):
        frame.paste(imgs[i], (10 + 100 * i, 200))

    try:
        frame.draw_text(
            (10, 300, frame.width - 10, 390),
            text,
            lines_align="center",
            weight="bold",
            max_fontsize=100,
            min_fontsize=30,
        )
    except ValueError:
        raise TextOverLength(text)

    return frame.save_jpg()


add_meme(
    "jiji_king",
    jiji_king,
    min_images=1,
    max_images=11,
    min_texts=0,
    max_texts=11,
    args_type=args_type,
    keywords=["急急国王"],
    date_created=datetime(2022, 10, 10),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import make_jpg_or_gif

img_dir = Path(__file__).parent / "images"


def pinch(images: list[BuildImage], texts, args):
    frame = BuildImage.open(img_dir / "0.png")

    def make(imgs: list[BuildImage]) -> BuildImage:
        return frame.paste(
            imgs[0].convert("RGBA").resize((1800, 1440), keep_ratio=True),
            (1080, 0),
            below=True,
        )

    return make_jpg_or_gif(images, make)


add_meme(
    "pinch",
    pinch,
    min_images=1,
    max_images=1,
    keywords=["捏", "捏脸"],
    date_created=datetime(2023, 11, 18),
    date_modified=datetime(2023, 11, 18),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import make_jpg_or_gif

img_dir = Path(__file__).parent / "images"


def think_what(images: list[BuildImage], texts, args):
    frame = BuildImage.open(img_dir / "0.png")

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA").resize((534, 493), keep_ratio=True)
        return frame.copy().paste(img, (530, 0), below=True)

    return make_jpg_or_gif(images, make)


add_meme(
    "think_what",
    think_what,
    min_images=1,
    max_images=1,
    keywords=["想什么"],
    date_created=datetime(2022, 5, 11),
    date_modified=datetime(2023, 2, 14),
)



import random
from datetime import datetime

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif


def turn(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").circle()
    frames: list[IMG] = []
    for i in range(0, 360, 10):
        frame = BuildImage.new("RGBA", (250, 250))
        frame.paste(img.rotate(i).resize((250, 250)), alpha=True)
        frames.append(frame.image)
    if random.randint(0, 1):
        frames.reverse()
    return save_gif(frames, 0.05)


add_meme(
    "turn",
    turn,
    min_images=1,
    max_images=1,
    keywords=["转"],
    date_created=datetime(2022, 1, 1),
    date_modified=datetime(2024, 9, 30),
)



import math
from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage, Text2Image

from meme_generator import add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"


def ace_attorney_dialog(images, texts: list[str], args):
    def shadow_text(text: str, fontsize: int) -> BuildImage:
        fontname = "PangMenZhengDao-Cu"
        inner = Text2Image.from_text(
            text,
            fontsize,
            fill="#e60012",
            fontname=fontname,
            stroke_width=4,
            stroke_fill="#500000",
        ).to_image()
        shadow_width = 10
        shadow = Text2Image.from_text(
            text,
            fontsize,
            fill="#500000",
            fontname=fontname,
            stroke_width=shadow_width,
            stroke_fill="#500000",
        ).to_image()
        dy = 30
        dx = 15
        img = BuildImage.new(
            "RGBA", (inner.width + dx + shadow_width, inner.height + dy + shadow_width)
        )
        img.paste(shadow, (dx - shadow_width, dy - shadow_width), alpha=True)
        img.paste(inner, (0, 0), alpha=True)
        return img

    text = texts[0]
    text_imgs: list[BuildImage] = []
    for char in text:
        text_imgs.append(shadow_text(char, 650))

    total_width = sum(img.width for img in text_imgs)
    if total_width > 4000:
        raise TextOverLength(text)

    def combine_text(text_imgs: list[BuildImage]) -> BuildImage:
        ratio = 0.4
        text_w = sum(img.width for img in text_imgs) - sum(
            round(img.width * ratio) for img in text_imgs[1:]
        )
        text_h = max(img.height for img in text_imgs)
        text_img = BuildImage.new("RGBA", (text_w, text_h))
        x = 0
        for img in text_imgs:
            text_img.paste(img, (x, round((text_h - img.height) / 2)), alpha=True)
            x += img.width - round(img.width * ratio)
        return text_img

    frame = BuildImage.open(img_dir / "bubble.png")
    mark = BuildImage.open(img_dir / "mark.png")

    if total_width <= 2000:
        text_img = combine_text(text_imgs)
        max_width = 900
        if total_width > max_width:
            text_img = text_img.resize(
                (max_width, round(max_width / text_img.width * text_img.height))
            )
        text_img = text_img.rotate(10, expand=True)
        frame.paste(
            text_img,
            (
                round((frame.width - text_img.width) / 2),
                round((frame.height - text_img.height) / 2),
            ),
            alpha=True,
        )
        frame.paste(mark, (630, 230), alpha=True)

    else:
        index = math.ceil(len(text_imgs) / 2)
        text_img1 = combine_text(text_imgs[:index])
        text_img2 = combine_text(text_imgs[index:])
        ratio = 0.6
        text_img1 = text_img1.resize(
            (round(text_img1.width * ratio), round(text_img1.height * ratio))
        )
        text_img2 = text_img2.resize(
            (round(text_img2.width * ratio), round(text_img2.height * ratio))
        )
        text_img1 = text_img1.rotate(10, expand=True)
        text_img2 = text_img2.rotate(10, expand=True)
        frame.paste(
            text_img1,
            (round((frame.width - text_img1.width) / 2) - 50, 775 - text_img1.height),
            alpha=True,
        )
        frame.paste(
            text_img2,
            (round((frame.width - text_img2.width) / 2) + 50, 325),
            alpha=True,
        )
        frame.paste(mark, (680, 320), alpha=True)

    return frame.save_png()


add_meme(
    "ace_attorney_dialog",
    ace_attorney_dialog,
    min_texts=1,
    max_texts=1,
    default_texts=["表情包制作"],
    keywords=["逆转裁判气泡"],
    date_created=datetime(2024, 5, 3),
    date_modified=datetime(2024, 5, 3),
)



from datetime import datetime
from typing import Literal

from arclet.alconna import store_value
from PIL.Image import Transpose
from pil_utils import BuildImage
from pydantic import Field

from meme_generator import (
    MemeArgsModel,
    MemeArgsType,
    ParserArg,
    ParserOption,
    add_meme,
)
from meme_generator.utils import FrameAlignPolicy, Maker, make_gif_or_combined_gif

help_text = "跑动方向，包含 left_right、right_left"


class Model(MemeArgsModel):
    direction: Literal["left_right", "right_left"] = Field(
        "left_right", description=help_text
    )


args_type = MemeArgsType(
    args_model=Model,
    args_examples=[Model(direction="left_right"), Model(direction="right_left")],
    parser_options=[
        ParserOption(
            names=["-d", "--direction"],
            args=[ParserArg(name="direction", value="str")],
            help_text=help_text,
        ),
        ParserOption(
            names=["--left_right", "左右"],
            dest="direction",
            action=store_value("left_right"),
        ),
        ParserOption(
            names=["--right_left", "右左"],
            dest="direction",
            action=store_value("right_left"),
        ),
    ],
)


def left_right_jump(images: list[BuildImage], texts, args: Model):
    img_w = 100
    img_h = images[0].resize_width(img_w).height
    frame_w = 300
    frame_h = img_h + 30
    frame = BuildImage.new("RGBA", (frame_w, frame_h))

    def traj(x: float) -> float:
        h = 15
        w = (frame_w - img_w) / 4
        k = h / w**2
        if x < img_w / 2 or x > frame_w - img_w / 2:
            return 0
        elif x < frame_w / 2:
            return h - k * (x - img_w / 2 - w) ** 2
        else:
            return h - k * (x - img_w / 2 - w * 3) ** 2

    frame_num = 30
    dx = (frame_w - img_w) / (frame_num / 2 - 1)

    def maker(i: int) -> Maker:
        def make(imgs: list[BuildImage]) -> BuildImage:
            img = imgs[0].convert("RGBA").resize_width(img_w)
            if args.direction == "left_right":
                if i >= round(frame_num / 2):
                    x = frame_w - img_w - dx * (frame_num - i - 1)
                    img = img.transpose(Transpose.FLIP_LEFT_RIGHT)
                else:
                    x = frame_w - img_w - dx * i
            else:
                if i >= round(frame_num / 2):
                    x = dx * (frame_num - i - 1)
                    img = img.transpose(Transpose.FLIP_LEFT_RIGHT)
                else:
                    x = dx * i
            y = frame_h - (traj(x + img_w / 2) + img_h)
            return frame.copy().paste(img, (round(x), round(y)), alpha=True)

        return make

    return make_gif_or_combined_gif(
        images, maker, frame_num, 0.04, FrameAlignPolicy.extend_loop
    )


add_meme(
    "left_right_jump",
    left_right_jump,
    min_images=1,
    max_images=1,
    args_type=args_type,
    keywords=["左右横跳"],
    date_created=datetime(2024, 7, 14),
    date_modified=datetime(2024, 7, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags
from meme_generator.utils import FrameAlignPolicy, Maker, make_gif_or_combined_gif

img_dir = Path(__file__).parent / "images"


def walnut_zoom(images: list[BuildImage], texts, args):
    # fmt: off
    locs = (
        (-222, 30, 695, 430), (-212, 30, 695, 430), (0, 30, 695, 430), (41, 26, 695, 430),
        (-100, -67, 922, 570), (-172, -113, 1059, 655), (-273, -192, 1217, 753)
    )
    seq = [0, 0, 0, 1, 2, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 4, 4, 4, 4, 5, 6, 6, 6, 6]
    # fmt: on

    def maker(i: int) -> Maker:
        def make(imgs: list[BuildImage]) -> BuildImage:
            frame = BuildImage.open(img_dir / f"{i}.png")
            x, y, w, h = locs[seq[i]]
            img = imgs[0].convert("RGBA").resize((w, h), keep_ratio=True)
            frame.paste(img.rotate(4.2, expand=True), (x, y), below=True)
            return frame

        return make

    return make_gif_or_combined_gif(
        images, maker, 24, 0.2, FrameAlignPolicy.extend_last
    )


add_meme(
    "walnut_zoom",
    walnut_zoom,
    min_images=1,
    max_images=1,
    keywords=["胡桃放大"],
    tags=MemeTags.walnut,
    date_created=datetime(2022, 10, 1),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"


def raise_sign(images, texts: list[str], args):
    text = texts[0]
    frame = BuildImage.open(img_dir / "0.jpg")
    text_img = BuildImage.new("RGBA", (360, 260))
    try:
        text_img.draw_text(
            (10, 10, 350, 250),
            text,
            max_fontsize=80,
            min_fontsize=30,
            allow_wrap=True,
            lines_align="center",
            spacing=10,
            fontname="FZShaoEr-M11S",
            fill="#51201b",
        )
    except ValueError:
        raise TextOverLength(text)
    text_img = text_img.perspective(((33, 0), (375, 120), (333, 387), (0, 258)))
    frame.paste(text_img, (285, 24), alpha=True)
    return frame.save_jpg()


add_meme(
    "raise_sign",
    raise_sign,
    min_texts=1,
    max_texts=1,
    default_texts=["大佬带带我"],
    keywords=["举牌"],
    date_created=datetime(2022, 6, 12),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import FrameAlignPolicy, Maker, make_gif_or_combined_gif

img_dir = Path(__file__).parent / "images"


def confuse(images: list[BuildImage], texts, args):
    img_w = min(images[0].width, 500)

    def maker(i: int) -> Maker:
        def make(imgs: list[BuildImage]) -> BuildImage:
            img = imgs[0].convert("RGBA").resize_width(img_w)
            frame = BuildImage.open(img_dir / f"{i}.png").resize(
                img.size, keep_ratio=True
            )
            bg = BuildImage.new("RGB", img.size, "white")
            bg.paste(img, alpha=True).paste(frame, alpha=True)
            return bg

        return make

    return make_gif_or_combined_gif(
        images, maker, 100, 0.02, FrameAlignPolicy.extend_loop
    )


add_meme(
    "confuse",
    confuse,
    min_images=1,
    max_images=1,
    keywords=["迷惑"],
    date_created=datetime(2022, 9, 4),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def wooden_fish(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").resize((85, 85))
    frames = [
        BuildImage.open(img_dir / f"{i}.png").paste(img, (116, 153), below=True).image
        for i in range(66)
    ]
    return save_gif(frames, 0.1)


add_meme(
    "wooden_fish",
    wooden_fish,
    min_images=1,
    max_images=1,
    keywords=["木鱼"],
    date_created=datetime(2022, 11, 16),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength
from meme_generator.tags import MemeTags
from meme_generator.utils import make_jpg_or_gif

img_dir = Path(__file__).parent / "images"

default_text = "所谓的男人啊，只要送他们这种东西就会很开心"


def frieren_take(images: list[BuildImage], texts: list[str], args):
    frame = BuildImage.open(img_dir / "0.png")
    text = texts[0] if texts else default_text
    try:
        frame.draw_text(
            (100, frame.height - 120, frame.width - 100, frame.height),
            text,
            max_fontsize=50,
            min_fontsize=20,
            fill="white",
            stroke_fill="black",
            stroke_ratio=0.05,
        )
    except ValueError:
        raise TextOverLength(text)

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA").resize((102, 108), keep_ratio=True)
        return frame.copy().paste(img, (130, 197), below=True)

    return make_jpg_or_gif(images, make)


add_meme(
    "frieren_take",
    frieren_take,
    min_images=1,
    max_images=1,
    max_texts=1,
    default_texts=[default_text],
    keywords=["芙莉莲拿"],
    tags=MemeTags.frieren,
    date_created=datetime(2024, 1, 18),
    date_modified=datetime(2024, 8, 9),
)



from datetime import datetime
from pathlib import Path
from typing import Literal

from arclet.alconna import store_value
from PIL.Image import Transpose
from pil_utils import BuildImage
from pydantic import Field

from meme_generator import (
    MemeArgsModel,
    MemeArgsType,
    ParserArg,
    ParserOption,
    add_meme,
)

img_dir = Path(__file__).parent / "images"


help_text = "枪的位置，包含 left、right、both"


class Model(MemeArgsModel):
    position: Literal["left", "right", "both"] = Field("left", description=help_text)


args_type = MemeArgsType(
    args_model=Model,
    args_examples=[
        Model(position="left"),
        Model(position="right"),
        Model(position="both"),
    ],
    parser_options=[
        ParserOption(
            names=["-p", "--position"],
            args=[ParserArg(name="position", value="str")],
            help_text=help_text,
        ),
        ParserOption(
            names=["--left", "左手"], dest="position", action=store_value("left")
        ),
        ParserOption(
            names=["--right", "右手"], dest="position", action=store_value("right")
        ),
        ParserOption(
            names=["--both", "双手"], dest="position", action=store_value("both")
        ),
    ],
)


def gun(images: list[BuildImage], texts, args: Model):
    frame = images[0].convert("RGBA").resize((500, 500), keep_ratio=True)
    gun = BuildImage.open(img_dir / "0.png")
    position = args.position
    left = position in ["left", "both"]
    right = position in ["right", "both"]
    if left:
        frame.paste(gun, alpha=True)
    if right:
        frame.paste(gun.transpose(Transpose.FLIP_LEFT_RIGHT), alpha=True)
    return frame.save_jpg()


add_meme(
    "gun",
    gun,
    min_images=1,
    max_images=1,
    args_type=args_type,
    keywords=["手枪"],
    date_created=datetime(2022, 8, 22),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme

img_dir = Path(__file__).parent / "images"


def back_to_work(images: list[BuildImage], texts, args):
    frame = BuildImage.open(img_dir / "0.png")
    img = (
        images[0].convert("RGBA").resize((220, 310), keep_ratio=True, direction="north")
    )
    frame.paste(img.rotate(25, expand=True), (56, 32), below=True)
    return frame.save_jpg()


add_meme(
    "back_to_work",
    back_to_work,
    min_images=1,
    max_images=1,
    keywords=["继续干活", "打工人"],
    date_created=datetime(2022, 3, 10),
    date_modified=datetime(2023, 2, 14),
)



import random
from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme

img_dir = Path(__file__).parent / "images"


def name_generator(images: list[BuildImage], texts, args):
    colors = ["#0000ff", "#ff00f7", "#00cc66"]
    # fmt: off
    el1 = ["废墟", "深海", "反应堆", "学园", "腐烂", "东京", "三维", "四次元", "少管所", "流星", "闪光", "南极", "消极", "幽浮", "网路", "暗狱", "离子态", "液态", "黑色", "抱抱", "暴力", "垃圾", "社会", "残暴", "残酷", "工口", "戮尸", "原味", "毛茸茸", "香香", "霹雳", "午夜", "美工刀", "爆浆", "机关枪", "无响应", "手术台", "麻风病", "虚拟", "速冻", "智能", "2000", "甜味", "华丽", "反社会", "玛利亚", "无", "梦之", "蔷薇", "无政府", "酷酷", "西伯利亚", "人造", "法外", "追杀", "通缉", "女子", "微型", "男子", "超", "毁灭", "大型", "绝望", "阴间", "死亡", "坟场", "高科技", "奇妙", "魔法", "极限", "社会主义", "无聊"]
    el2 = ["小丑", "仿生", "纳米", "原子", "丧", "电子", "十字架", "咩咩", "赛博", "野猪", "外星", "窒息", "变态", "触手", "小众", "悲情", "飞行", "绿色", "电动", "铁锈", "碎尸", "电音", "蠕动", "酸甜", "虚构", "乱码", "碳水", "内脏", "脑浆", "血管", "全裸", "绷带", "不合格", "光滑", "标本", "酸性", "碱性", "404", "变身", "反常", "樱桃", "碳基", "矫情", "病娇", "进化", "潮湿", "砂糖", "高潮", "变异", "复合盐", "伏特加", "抑郁", "暴躁", "不爱说话", "废物", "失败", "幻想型", "社恐", "苦涩", "粘液", "浓厚", "快乐", "强制", "中二病", "恶魔", "emo", "激光", "发射", "限量版", "迷因", "堕落", "放射性"]
    el3 = ["天使", "精灵", "女孩", "男孩", "宝贝", "小妈咪", "虫", "菇", "公主", "少女", "少年", "1号机", "子", "恐龙", "蜈蚣", "蟑螂", "食人鱼", "小飞船", "舞女", "桃子", "团子", "精", "酱", "废料", "生物", "物质", "奶茶", "搅拌机", "液", "火锅", "祭司", "体", "实验品", "试验体", "小猫咪", "样本", "颗粒", "血块", "汽水", "蛙", "软体", "机器人", "人质", "小熊", "圣母", "胶囊", "乙女", "主义者", "屑", "垢", "污渍", "废人", "毛血旺", "怪人", "肉", "河豚", "豚", "藻类", "唾沫", "咒语", "建筑", "球", "小狗", "碳", "元素", "少先队员", "博士", "糖"]
    # fmt: on
    color = random.choice(colors)
    name = random.choice(el1) + random.choice(el2) + random.choice(el3)
    frame = BuildImage.new("RGB", (900, 900), (225, 225, 225))
    title = BuildImage.open(img_dir / "title.png").resize((700, 200))
    frame.paste(title, (100, 0), alpha=True)
    img = images[0].convert("RGBA").resize((490, 490), keep_ratio=True)
    frame.paste(img, (310, 235), alpha=True)
    frame.draw_text(
        (20, 450), "您的亚名是", fontsize=40, fill=(105, 105, 105), weight="bold"
    ).draw_text(
        (20, 620),
        name,
        fontsize=60,
        fill=color,
        weight="bold",
        stroke_ratio=0.07,
        stroke_fill="white",
    )
    return frame.save_jpg()


add_meme(
    "name_generator",
    name_generator,
    min_images=1,
    max_images=1,
    keywords=["亚文化取名机", "亚名"],
    date_created=datetime(2023, 2, 4),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import MemeArgsModel, add_meme
from meme_generator.exception import TextOverLength
from meme_generator.utils import make_jpg_or_gif

img_dir = Path(__file__).parent / "images"


def stew(images: list[BuildImage], texts: list[str], args: MemeArgsModel):
    name = texts[0] if texts else args.user_infos[0].name if args.user_infos else "群友"
    text = f"生活不易,炖{name}出气"

    frame = BuildImage.open(img_dir / "0.png")
    try:
        frame.draw_text(
            (2, frame.height - 30, frame.width - 2, frame.height),
            text,
            max_fontsize=30,
            min_fontsize=6,
            lines_align="center",
        )
    except ValueError:
        raise TextOverLength(name)

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA").resize((181, 154), keep_ratio=True)
        return frame.copy().paste(img, (9, -2), below=True)

    return make_jpg_or_gif(images, make)


add_meme(
    "stew",
    stew,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    keywords=["炖"],
    date_created=datetime(2024, 1, 19),
    date_modified=datetime(2024, 1, 19),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def printing(images: list[BuildImage], texts, args):
    img = (
        images[0]
        .convert("RGBA")
        .resize(
            (304, 174),
            keep_ratio=True,
            inside=True,
            bg_color="white",
            direction="south",
        )
    )
    frames = [BuildImage.open(img_dir / f"{i}.png") for i in range(115)]
    for i in range(50, 115):
        frames[i].paste(img, (146, 164), below=True)
    frames = [frame.image for frame in frames]
    return save_gif(frames, 0.05)


add_meme(
    "printing",
    printing,
    min_images=1,
    max_images=1,
    keywords=["打印"],
    date_created=datetime(2023, 1, 26),
    date_modified=datetime(2023, 2, 14),
)



class MemeTags:
    # 米家游戏
    mihoyo = {"米哈游"}

    genshin = {"原神"} | mihoyo
    star_rail = {"崩坏：星穹铁道"} | mihoyo
    honkai3 = {"崩坏3"} | mihoyo

    nahida = {"纳西妲", "草神"} | genshin
    hutao = {"胡桃"} | genshin
    klee = {"可莉"} | genshin
    keqing = {"刻晴"} | genshin
    zhongli = {"钟离"} | genshin
    nilou = {"妮露"} | genshin
    yae_miko = {"八重神子"} | genshin
    ayaka = {"神里绫华"} | genshin
    bronya = {"布洛妮娅·扎伊切克"} | honkai3
    captain = {"休伯利安号", "舰长"} | honkai3
    griseo = {"格蕾修"} | honkai3
    firefly = {"流萤"} | star_rail

    # 蔚蓝档案
    blue_archive = {"蔚蓝档案", "碧蓝档案"}

    shiroko = {"砂狼白子"} | blue_archive

    # 公主连结
    re_dive = {"公主连结"}

    karyl = {"凯露"} | re_dive

    # 间谍过家家
    spy_family = {"间谍过家家"}

    anya = {"阿尼亚·福杰"} | spy_family

    # 孤独摇滚
    bocchi_the_rock = {"孤独摇滚"}

    bocchi = {"后藤独", "后藤一里", "波奇酱"} | bocchi_the_rock
    nijika = {"伊地知虹夏"} | bocchi_the_rock

    # 咒术回战
    jujutsu_kaisen = {"咒术回战"}

    sukuna = {"两面宿傩"} | jujutsu_kaisen

    # 葬送的芙莉莲
    sousou_no_frieren = {"葬送的芙莉莲"}

    frieren = {"芙莉莲"} | sousou_no_frieren

    # 我推的孩子
    oshi_no_ko = {"我推的孩子"}

    # LoveLive!Superstar!!
    lovelive_superstar = {"LoveLive!Superstar!!"}

    tan_kuku = {"唐可可"} | lovelive_superstar

    # 莉可丽丝
    lycoris_recoil = {"莉可丽丝"}

    takina = {"井之上泷奈"} | lycoris_recoil
    walnut = {"胡桃"} | lycoris_recoil

    # 别当欧尼酱了
    onimai = {"别当欧尼酱了"}

    mahiro = {"绪山真寻"} | onimai

    # 幸运星
    lucky_star = {"幸运星"}

    konata = {"泉此方"} | lucky_star

    # 猫和老鼠
    tom_and_jerry = {"猫和老鼠"}

    tom = {"汤姆"} | tom_and_jerry
    jerry = {"杰瑞"} | tom_and_jerry

    # 瑞克和莫蒂
    rick_and_morty = {"瑞克和莫蒂"}

    rick = {"瑞克·桑切斯"} | rick_and_morty

    # VTuber
    gura = {"噶呜·古拉", "Gawr Gura", "鲨鲨"}

    # VOCALOID
    miku = {"初音未来"}

    # 明日方舟
    arknights = {"明日方舟"}

    # 东方Project
    touhou = {"东方Project"}

    # 舞萌
    maimai = {"舞萌"}

    # 哈利·波特
    harry_potter = {"哈利·波特"}

    # 其他
    capoo = {"猫猫虫", "咖波"}
    nekoha = {"猫羽雫"}
    kirby = {"星之卡比"}
    atri = {"亚托莉", "ATRI", "萝卜子"}



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags
from meme_generator.utils import make_png_or_gif

img_dir = Path(__file__).parent / "images"


def jerry_stare(images: list[BuildImage], texts, args):
    jerry = BuildImage.open(img_dir / "0.png").convert("RGBA")

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA").circle().resize((150, 150), keep_ratio=True)
        return jerry.copy().paste(img, (184, 268), below=True)

    return make_png_or_gif(images, make)


add_meme(
    "jerry_stare",
    jerry_stare,
    min_images=1,
    max_images=1,
    keywords=["杰瑞盯"],
    tags=MemeTags.jerry,
    date_created=datetime(2024, 8, 9),
    date_modified=datetime(2024, 8, 9),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import CommandShortcut, add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"


def high_EQ(images, texts: list[str], args):
    frame = BuildImage.open(img_dir / "0.jpg")

    def draw(pos: tuple[float, float, float, float], text: str):
        try:
            frame.draw_text(
                pos,
                text,
                max_fontsize=100,
                min_fontsize=50,
                allow_wrap=True,
                fill="white",
                stroke_fill="black",
                stroke_ratio=0.05,
            )
        except ValueError:
            raise TextOverLength(text)

    draw((40, 540, 602, 1140), texts[0])
    draw((682, 540, 1244, 1140), texts[1])
    return frame.save_jpg()


add_meme(
    "high_EQ",
    high_EQ,
    min_texts=2,
    max_texts=2,
    default_texts=["高情商", "低情商"],
    keywords=["高低情商", "低高情商"],
    shortcuts=[
        CommandShortcut(
            key=r"低情商[\s:：]*(?P<low>\S+)\s*高情商[\s:：]*(?P<high>\S+)",
            args=["{low}", "{high}"],
            humanized="低情商xx高情商xx",
        ),
        CommandShortcut(
            key=r"高情商[\s:：]*(?P<high>\S+)\s*低情商[\s:：]*(?P<low>\S+)",
            args=["{low}", "{high}"],
            humanized="高情商xx低情商xx",
        ),
    ],
    date_created=datetime(2022, 6, 12),
    date_modified=datetime(2024, 8, 12),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"


def find_chips(images, texts: list[str], args):
    frame = BuildImage.open(img_dir / "0.jpg")

    def draw(pos: tuple[float, float, float, float], text: str):
        try:
            frame.draw_text(
                pos, text, max_fontsize=30, min_fontsize=12, allow_wrap=True
            )
        except ValueError:
            raise TextOverLength(text)

    draw((405, 54, 530, 130), texts[0])
    draw((570, 62, 667, 160), texts[1])
    draw((65, 400, 325, 463), texts[2])
    draw((430, 400, 630, 470), texts[3])
    return frame.save_jpg()


add_meme(
    "find_chips",
    find_chips,
    min_texts=4,
    max_texts=4,
    default_texts=[
        "我们要飞向何方",
        "我打算待会去码头整点薯条",
        "我说的是归根结底，活着是为了什么",
        "为了待会去码头整点薯条",
    ],
    keywords=["整点薯条"],
    date_created=datetime(2022, 10, 26),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme

img_dir = Path(__file__).parent / "images"


def hold_tight(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").resize((159, 171), keep_ratio=True)
    frame = BuildImage.open(img_dir / "0.png")
    frame.paste(img, (113, 205), below=True)
    return frame.save_jpg()


add_meme(
    "hold_tight",
    hold_tight,
    min_images=1,
    max_images=1,
    keywords=["抱紧"],
    date_created=datetime(2022, 10, 1),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage, Text2Image

from meme_generator import add_meme
from meme_generator.exception import TextOverLength
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def scroll(images, texts: list[str], args):
    text = texts[0]
    text2image = Text2Image.from_text(text, 40).wrap(600)
    if len(text2image.lines) > 5:
        raise TextOverLength(text)
    text_img = text2image.to_image()
    text_w, text_h = text_img.size

    box_w = text_w + 140
    box_h = max(text_h + 103, 150)
    box = BuildImage.new("RGBA", (box_w, box_h), "#eaedf4")
    corner1 = BuildImage.open(img_dir / "corner1.png")
    corner2 = BuildImage.open(img_dir / "corner2.png")
    corner3 = BuildImage.open(img_dir / "corner3.png")
    corner4 = BuildImage.open(img_dir / "corner4.png")
    box.paste(corner1, (0, 0))
    box.paste(corner2, (0, box_h - 75))
    box.paste(corner3, (text_w + 70, 0))
    box.paste(corner4, (text_w + 70, box_h - 75))
    box.paste(BuildImage.new("RGBA", (text_w, box_h - 40), "white"), (70, 20))
    box.paste(BuildImage.new("RGBA", (text_w + 88, box_h - 150), "white"), (27, 75))
    box.paste(text_img, (70, 17 + (box_h - 40 - text_h) // 2), alpha=True)

    dialog = BuildImage.new("RGBA", (box_w, box_h * 4), "#eaedf4")
    for i in range(4):
        dialog.paste(box, (0, box_h * i))

    frames: list[IMG] = []
    num = 30
    dy = int(dialog.height / num)
    for i in range(num):
        frame = BuildImage.new("RGBA", dialog.size)
        frame.paste(dialog, (0, -dy * i))
        frame.paste(dialog, (0, dialog.height - dy * i))
        frames.append(frame.image)
    return save_gif(frames, 0.05)


add_meme(
    "scroll",
    scroll,
    min_texts=1,
    max_texts=1,
    default_texts=["你们说话啊"],
    keywords=["滚屏"],
    date_created=datetime(2022, 1, 19),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def fencing(images: list[BuildImage], texts, args):
    self_head = images[0].convert("RGBA").circle().resize((27, 27))
    user_head = images[1].convert("RGBA").circle().resize((27, 27))
    # fmt: off
    user_locs = [
        (57, 4), (55, 5), (58, 7), (57, 5), (53, 8), (54, 9),
        (64, 5), (66, 8), (70, 9), (73, 8), (81, 10), (77, 10),
        (72, 4), (79, 8), (50, 8), (60, 7), (67, 6), (60, 6), (50, 9)
    ]
    self_locs = [
        (10, 6), (3, 6), (32, 7), (22, 7), (13, 4), (21, 6),
        (30, 6), (22, 2), (22, 3), (26, 8), (23, 8), (27, 10),
        (30, 9), (17, 6), (12, 8), (11, 7), (8, 6), (-2, 10), (4, 9)
    ]
    # fmt: on
    frames: list[IMG] = []
    for i in range(19):
        frame = BuildImage.open(img_dir / f"{i}.png")
        frame.paste(user_head, user_locs[i], alpha=True)
        frame.paste(self_head, self_locs[i], alpha=True)
        frames.append(frame.image)
    return save_gif(frames, 0.05)


add_meme(
    "fencing",
    fencing,
    min_images=2,
    max_images=2,
    keywords=["击剑", "🤺"],
    date_created=datetime(2022, 10, 1),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from PIL import ImageFilter
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import make_jpg_or_gif

img_dir = Path(__file__).parent / "images"


def loading(images: list[BuildImage], texts, args):
    img_big = images[0].convert("RGBA").resize_width(500)
    img_big = img_big.filter(ImageFilter.GaussianBlur(radius=3))
    h1 = img_big.height
    mask = BuildImage.new("RGBA", img_big.size, (0, 0, 0, 32))
    icon = BuildImage.open(img_dir / "icon.png")
    img_big.paste(mask, alpha=True).paste(icon, (200, int(h1 / 2) - 50), alpha=True)

    def make(imgs: list[BuildImage]) -> BuildImage:
        img_small = imgs[0].convert("RGBA").resize_width(100)
        h2 = max(img_small.height, 80)
        frame = BuildImage.new("RGBA", (500, h1 + h2 + 10), "white")
        frame.paste(img_big, alpha=True).paste(
            img_small, (100, h1 + 5 + (h2 - img_small.height) // 2), alpha=True
        )
        frame.draw_text(
            (210, h1 + 5, 480, h1 + h2 + 5), "不出来", halign="left", max_fontsize=60
        )
        return frame

    return make_jpg_or_gif(images, make)


add_meme(
    "loading",
    loading,
    min_images=1,
    max_images=1,
    keywords=["加载中"],
    date_created=datetime(2021, 12, 29),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage, Text2Image

from meme_generator import MemeArgsModel, add_meme
from meme_generator.exception import TextOverLength
from meme_generator.utils import random_text, save_gif

img_dir = Path(__file__).parent / "images"


def repeat(images: list[BuildImage], texts: list[str], args: MemeArgsModel):
    def single_msg(img: BuildImage, name: str) -> BuildImage:
        user_img = img.convert("RGBA").circle().resize((100, 100))
        user_name_img = Text2Image.from_text(f"{name}", 40).to_image()
        time = datetime.now().strftime("%H:%M")
        time_img = Text2Image.from_text(time, 40, fill="gray").to_image()
        bg = BuildImage.new("RGB", (1079, 200), (248, 249, 251, 255))
        bg.paste(user_img, (50, 50), alpha=True)
        bg.paste(user_name_img, (175, 45), alpha=True)
        bg.paste(time_img, (200 + user_name_img.width, 50), alpha=True)
        bg.paste(text_img, (175, 100), alpha=True)
        return bg

    text = texts[0]
    text_img = Text2Image.from_text(text, 50).to_image()
    if text_img.width > 900:
        raise TextOverLength(text)

    users: list[tuple[BuildImage, str]] = []
    user_infos = args.user_infos
    for i, image in enumerate(images):
        name = user_infos[i].name if len(user_infos) > i else random_text()
        users.append((image, name))

    msg_img = BuildImage.new("RGB", (1079, 1000))
    for i in range(5):
        index = i % len(users)
        msg_img.paste(single_msg(*users[index]), (0, 200 * i))
    msg_img_twice = BuildImage.new("RGB", (msg_img.width, msg_img.height * 2))
    msg_img_twice.paste(msg_img).paste(msg_img, (0, msg_img.height))

    input_img = BuildImage.open(img_dir / "0.jpg")
    self_img = images[0].convert("RGBA").circle().resize((75, 75))
    input_img.paste(self_img, (15, 40), alpha=True)

    frames: list[IMG] = []
    for i in range(50):
        frame = BuildImage.new("RGB", (1079, 1192), "white")
        frame.paste(msg_img_twice, (0, -20 * i))
        frame.paste(input_img, (0, 1000))
        frames.append(frame.image)

    return save_gif(frames, 0.08)


add_meme(
    "repeat",
    repeat,
    min_images=1,
    max_images=5,
    min_texts=1,
    max_texts=1,
    default_texts=["救命啊"],
    keywords=["复读"],
    date_created=datetime(2022, 6, 8),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength
from meme_generator.utils import make_jpg_or_gif

default_text = "我看你们是反了！"


def upside_down(images: list[BuildImage], texts: list[str], args):
    text = texts[0] if texts else default_text

    img_w = 500
    text_h = 80
    text_frame = BuildImage.new("RGBA", (img_w, text_h), "white")
    try:
        text_frame.draw_text(
            (20, 0, img_w - 20, text_h),
            text,
            max_fontsize=55,
            min_fontsize=30,
        )
    except ValueError:
        raise TextOverLength(text)

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA").resize_width(img_w).rotate(180)
        img_h = img.height
        frame = BuildImage.new("RGBA", (img_w, img_h + text_h), "white")
        return frame.paste(text_frame, alpha=True).paste(img, (0, text_h), alpha=True)

    return make_jpg_or_gif(images, make)


add_meme(
    "upside_down",
    upside_down,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    default_texts=[default_text],
    keywords=["反了"],
    date_created=datetime(2024, 10, 12),
    date_modified=datetime(2024, 10, 12),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def tightly(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").resize((640, 400), keep_ratio=True)
    # fmt: off
    locs = [(39, 169, 267, 141), (40, 167, 264, 143), (38, 174, 270, 135), (40, 167, 264, 143), (38, 174, 270, 135),
            (40, 167, 264, 143), (38, 174, 270, 135), (40, 167, 264, 143), (38, 174, 270, 135), (28, 176, 293, 134),
            (5, 215, 333, 96), (10, 210, 321, 102), (3, 210, 330, 104), (4, 210, 328, 102), (4, 212, 328, 100),
            (4, 212, 328, 100), (4, 212, 328, 100), (4, 212, 328, 100), (4, 212, 328, 100), (29, 195, 285, 120)]
    # fmt: on
    frames: list[IMG] = []
    for i in range(20):
        frame = BuildImage.open(img_dir / f"{i}.png")
        x, y, w, h = locs[i]
        frame.paste(img.resize((w, h)), (x, y), below=True)
        frames.append(frame.image)
    return save_gif(frames, 0.08)


add_meme(
    "tightly",
    tightly,
    min_images=1,
    max_images=1,
    keywords=["紧贴", "紧紧贴着"],
    date_created=datetime(2022, 4, 20),
    date_modified=datetime(2023, 2, 14),
)



import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags
from meme_generator.utils import FrameAlignPolicy, Maker, make_gif_or_combined_gif

img_dir = Path(__file__).parent / "images"


def capoo_stew(images: list[BuildImage], texts, args):
    def maker(i: int) -> Maker:
        def make(imgs: list[BuildImage]) -> BuildImage:
            img = imgs[0].convert("RGBA").resize((80, 80), keep_ratio=True).circle()
            bg = BuildImage.open(img_dir / f"{i}.png")
            if i in [2, 3, 5]:
                y = 45
            else:
                y = 47
            return bg.paste(img, (88, y), below=True)

        return make

    return make_gif_or_combined_gif(
        images, maker, 5, 0.08, FrameAlignPolicy.extend_loop
    )


add_meme(
    "capoo_stew",
    capoo_stew,
    max_images=1,
    min_images=1,
    tags=MemeTags.capoo,
    keywords=["咖波炖"],
    date_created=datetime.datetime(2024, 8, 23),
    date_modified=datetime.datetime(2024, 8, 23),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength
from meme_generator.utils import make_png_or_gif

img_dir = Path(__file__).parent / "images"

defalut_text = "笨蛋"


def thermometer_gun(images: list[BuildImage], texts: list[str], args):
    text = defalut_text if not len(texts) else texts[0]
    text_frame = BuildImage.new("RGBA", (200, 125))
    try:
        text_frame.draw_text(
            (0, 0, 200, 125),
            text,
            allow_wrap=True,
            max_fontsize=60,
            min_fontsize=15,
            fontname="FZKaTong-M19S",
            lines_align="center",
        )
    except ValueError:
        raise TextOverLength(text)
    img_w, img_h = images[0].size
    if img_w > img_h:
        size = (round(img_h), round(img_h))
        pos = (round(img_w - img_h), 0)
    else:
        size = (round(img_w), round(img_w))
        pos = (0, round(img_h - img_w))
    frame = (
        BuildImage.open(img_dir / "0.png")
        .paste(text_frame, (555, 240), alpha=True)
        .resize(size)
    )

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA")
        return img.paste(frame, pos, alpha=True)

    return make_png_or_gif(images, make)


add_meme(
    "thermometer_gun",
    thermometer_gun,
    min_images=1,
    max_images=1,
    max_texts=1,
    min_texts=0,
    keywords=["体温枪"],
    default_texts=[defalut_text],
    date_created=datetime(2024, 9, 3),
    date_modified=datetime(2024, 9, 3),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from PIL.Image import Resampling, Transform
from pil_utils import BuildImage, Text2Image
from pil_utils.fonts import DEFAULT_FALLBACK_FONTS
from pil_utils.text2image import Line

from meme_generator import add_meme
from meme_generator.tags import MemeTags

img_dir = Path(__file__).parent / "images"


def bluearchive(images, texts: list[str], args):
    fontsize = 168
    fontname = "Ro GSan Serif Std"
    fallback_fonts = ["Glow Sans SC"] + DEFAULT_FALLBACK_FONTS
    tilt = 0.4
    color_blue = "#128AFA"
    color_gray = "#2B2B2B"

    def transform(img: IMG) -> IMG:
        dw = round(img.height * tilt)
        return img.transform(
            (img.width + dw, img.height),
            Transform.AFFINE,
            (1, tilt, -dw, 0, 1, 0),
            Resampling.BILINEAR,
        )

    left_t2m = Text2Image.from_text(
        texts[0],
        fontsize,
        fill=color_blue,
        fontname=fontname,
        fallback_fonts=fallback_fonts,
    )
    right_t2m = Text2Image.from_text(
        texts[1],
        fontsize,
        fill=color_gray,
        stroke_width=12,
        stroke_fill="white",
        fontname=fontname,
        fallback_fonts=fallback_fonts,
    )
    new_line = Line(
        left_t2m.lines[0].chars + right_t2m.lines[0].chars,
        fontsize=fontsize,
        fontname=fontname,
    )
    text_t2m = Text2Image([new_line])
    text_img = transform(text_t2m.to_image())
    text_dy = text_t2m.lines[0].ascent

    padding_x = 50
    img_w = text_img.width + padding_x * 2
    img_h = 450
    text_y = 350
    logo_y = 10
    logo_x = padding_x + left_t2m.width - 115
    halo = BuildImage.open(img_dir / "halo.png").convert("RGBA")
    cross = BuildImage.open(img_dir / "cross.png").convert("RGBA")

    frame = BuildImage.new("RGBA", (img_w, img_h), (255, 255, 255, 0))
    frame.paste(halo, (logo_x, logo_y), alpha=True)
    frame.paste(text_img, (padding_x, text_y - text_dy), alpha=True)
    frame.paste(cross, (logo_x, logo_y), alpha=True)
    return frame.save_jpg()


add_meme(
    "bluearchive",
    bluearchive,
    min_texts=2,
    max_texts=2,
    default_texts=["Blue", "Archive"],
    keywords=["蔚蓝档案标题", "batitle"],
    tags=MemeTags.blue_archive,
    date_created=datetime(2023, 10, 14),
    date_modified=datetime(2023, 10, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import MemeArgsModel, add_meme
from meme_generator.exception import TextOrNameNotEnough, TextOverLength

img_dir = Path(__file__).parent / "images"


def why_have_hands(images: list[BuildImage], texts: list[str], args: MemeArgsModel):
    img = images[0].convert("RGBA")

    if not texts and not args.user_infos:
        raise TextOrNameNotEnough()
    name = texts[0] if texts else args.user_infos[0].name

    frame = BuildImage.open(img_dir / "0.png")
    frame.paste(img.circle().resize((250, 250)), (350, 670), alpha=True)
    frame.paste(
        img.resize((250, 250), keep_ratio=True).rotate(15), (1001, 668), below=True
    )
    frame.paste(img.resize((250, 170), keep_ratio=True), (275, 1100), below=True)
    frame.paste(
        img.resize((300, 400), keep_ratio=True, inside=True, direction="west"),
        (1100, 1060),
        alpha=True,
    )
    try:
        text_frame = BuildImage.new("RGBA", (600, 100)).draw_text(
            (0, 0, 600, 100),
            f"摸摸{name}!",
            max_fontsize=70,
            min_fontsize=30,
            halign="left",
            weight="bold",
        )
        frame.paste(text_frame.rotate(-15, expand=True), (75, 825), alpha=True)
        frame.draw_text(
            (840, 960, 1440, 1060),
            f"托托{name}!",
            max_fontsize=70,
            min_fontsize=30,
            weight="bold",
        )
        frame.draw_text(
            (50, 1325, 650, 1475),
            f"赞美{name}!",
            max_fontsize=90,
            min_fontsize=30,
            weight="bold",
            valign="top",
        )
        frame.draw_text(
            (700, 1340, 1075, 1490),
            f"为{name}奉献所有财产!",
            max_fontsize=70,
            min_fontsize=30,
            weight="bold",
            allow_wrap=True,
        )
    except ValueError:
        raise TextOverLength(name)

    return frame.save_jpg()


add_meme(
    "why_have_hands",
    why_have_hands,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    keywords=["为什么要有手"],
    date_created=datetime(2023, 5, 18),
    date_modified=datetime(2023, 5, 18),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme

img_dir = Path(__file__).parent / "images"


def rip_angrily(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").square().resize((105, 105))
    frame = BuildImage.open(img_dir / "0.png")
    frame.paste(img.rotate(-24, expand=True), (18, 170), below=True)
    frame.paste(img.rotate(24, expand=True), (163, 65), below=True)
    return frame.save_jpg()


add_meme(
    "rip_angrily",
    rip_angrily,
    min_images=1,
    max_images=1,
    keywords=["怒撕"],
    date_created=datetime(2022, 10, 9),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"

default_text = "怎么说话的你"


def beat_head(images: list[BuildImage], texts: list[str], args):
    text = texts[0] if texts else default_text
    img = images[0].convert("RGBA")
    locs = [(160, 121, 76, 76), (172, 124, 69, 69), (208, 166, 52, 52)]
    frames: list[IMG] = []
    for i in range(3):
        x, y, w, h = locs[i]
        head = img.resize((w, h), keep_ratio=True).circle()
        frame = BuildImage.open(img_dir / f"{i}.png")
        frame.paste(head, (x, y), below=True)
        try:
            frame.draw_text(
                (175, 28, 316, 82),
                text,
                max_fontsize=50,
                min_fontsize=10,
                allow_wrap=True,
            )
        except ValueError:
            raise TextOverLength(text)

        frames.append(frame.image)
    return save_gif(frames, 0.05)


add_meme(
    "beat_head",
    beat_head,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    default_texts=[default_text],
    keywords=["拍头"],
    date_created=datetime(2023, 3, 8),
    date_modified=datetime(2023, 3, 8),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def twist(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").square().resize((78, 78))
    # fmt: off
    locs = [
        (25, 66, 0), (25, 66, 60), (23, 68, 120),
        (20, 69, 180), (22, 68, 240), (25, 66, 300)
    ]
    # fmt: on
    frames: list[IMG] = []
    for i in range(5):
        frame = BuildImage.open(img_dir / f"{i}.png")
        x, y, a = locs[i]
        frame.paste(img.rotate(a), (x, y), below=True)
        frames.append(frame.image)
    return save_gif(frames, 0.1)


add_meme(
    "twist",
    twist,
    min_images=1,
    max_images=1,
    keywords=["搓"],
    date_created=datetime(2022, 3, 9),
    date_modified=datetime(2023, 2, 14),
)



import random
from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme

img_dir = Path(__file__).parent / "images"


def throw(images: list[BuildImage], texts, args):
    img = (
        images[0]
        .convert("RGBA")
        .circle()
        .rotate(random.randint(1, 360))
        .resize((143, 143))
    )
    frame = BuildImage.open(img_dir / "0.png")
    frame.paste(img, (15, 178), alpha=True)
    return frame.save_jpg()


add_meme(
    "throw",
    throw,
    min_images=1,
    max_images=1,
    keywords=["丢", "扔"],
    date_created=datetime(2021, 5, 5),
    date_modified=datetime(2023, 3, 30),
)



from datetime import datetime
from pathlib import Path
from typing import Literal

from pil_utils import BuildImage

from meme_generator import CommandShortcut, add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"


def wujing(images, texts: list[str], args):
    frame = BuildImage.open(img_dir / "0.jpg")

    def draw(
        pos: tuple[float, float, float, float],
        text: str,
        align: Literal["left", "right", "center"],
    ):
        try:
            frame.draw_text(
                pos,
                text,
                halign=align,
                max_fontsize=100,
                min_fontsize=50,
                fill="white",
                stroke_fill="black",
                stroke_ratio=0.05,
            )
        except ValueError:
            raise TextOverLength(text)

    if texts[0]:
        parts = texts[0].split()
        if len(parts) >= 2:
            draw((50, 430, 887, 550), " ".join(parts[:-1]), "left")
        draw((20, 560, 350, 690), parts[-1], "right")
    if texts[1]:
        parts = texts[1].split()
        draw((610, 540, 917, 670), parts[0], "left")
        if len(parts) >= 2:
            draw((50, 680, 887, 810), " ".join(parts[1:]), "center")

    return frame.save_jpg()


add_meme(
    "wujing",
    wujing,
    min_texts=2,
    max_texts=2,
    default_texts=["不买华为不是", "人"],
    keywords=["吴京中国"],
    shortcuts=[
        CommandShortcut(
            key=r"吴京[\s:：]*(?P<left>\S*)中国(?P<right>\S*)",
            args=["{left}", "{right}"],
            humanized="吴京xx中国xx",
        )
    ],
    date_created=datetime(2022, 6, 12),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"


def nijika_holdsign(images, texts: list[str], args):
    text = texts[0]
    frame = BuildImage.open(img_dir / "0.jpg")
    try:
        frame.draw_text(
            (57, 279, 249, 405),
            text,
            fill=(111, 95, 95),
            allow_wrap=True,
            max_fontsize=60,
            min_fontsize=25,
            lines_align="center",
            fontname="FZSJ-QINGCRJ",
        )
    except ValueError:
        raise TextOverLength(text)
    return frame.save_jpg()


add_meme(
    "nijika_holdsign",
    nijika_holdsign,
    min_texts=1,
    max_texts=1,
    default_texts=["你可少看点二次元吧"],
    keywords=["伊地知虹夏举牌", "虹夏举牌"],
    date_created=datetime(2023, 6, 20),
    date_modified=datetime(2023, 6, 20),
)



from datetime import datetime

from pil_utils import BuildImage

from meme_generator import MemeArgsModel, add_meme
from meme_generator.exception import TextOrNameNotEnough, TextOverLength


def google_captcha(images: list[BuildImage], texts: list[str], args: MemeArgsModel):
    if not texts and not args.user_infos:
        raise TextOrNameNotEnough()

    name = texts[0] if texts else args.user_infos[0].name
    image = images[0].convert("RGBA").resize((932, 932), keep_ratio=True)

    canvas = BuildImage.new("RGB", (1000, 1535), "#FFF")
    length = 233
    for i in range(4):
        for j in range(4):
            box = (length * i, length * j, length * (i + 1), length * (j + 1))
            canvas.paste(image.crop(box), (19 + i * (233 + 10), 370 + j * (233 + 10)))

    banner = BuildImage.new("RGB", (962, 332), "#4790E4")
    banner.draw_text(
        (70, 60, 900, 120),
        "请选择包含",
        fill="white",
        max_fontsize=40,
        weight="bold",
        halign="left",
    )
    try:
        banner.draw_text(
            (70, 120, 900, 210),
            name,
            fill="white",
            max_fontsize=80,
            weight="bold",
            halign="left",
        )
    except ValueError:
        raise TextOverLength(name)
    banner.draw_text(
        (70, 210, 900, 270),
        "的所有图块，如果没有，请点击“跳过”",
        fill="white",
        max_fontsize=40,
        weight="bold",
        halign="left",
    )
    canvas.paste(banner, (19, 19))

    bottom = BuildImage.new("RGB", (1000, 182), "#FFF")
    button = BuildImage.new("RGB", (283, 121), "#4790E4")
    button.draw_text(
        (0, 0, 283, 121), "跳过", fill="white", max_fontsize=40, weight="bold"
    )
    bottom.paste(button, (687, 30))
    bottom_border = BuildImage.new("RGB", (1002, 186), "#D5D5D5")
    bottom_border.paste(bottom, (2, 2))
    canvas.paste(bottom_border, (-2, 1353))

    result = BuildImage.new("RGB", (1004, 1539), "#D5D5D5")
    result.paste(canvas, (2, 2))
    return result.save_jpg()


add_meme(
    "google_captcha",
    google_captcha,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    keywords=["谷歌验证码"],
    date_created=datetime(2024, 8, 15),
    date_modified=datetime(2024, 8, 15),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"


def good_news(images, texts: list[str], args):
    text = texts[0]
    frame = BuildImage.open(img_dir / "0.jpg")
    try:
        frame.draw_text(
            (50, 100, frame.width - 50, frame.height - 100),
            text,
            allow_wrap=True,
            lines_align="center",
            max_fontsize=60,
            min_fontsize=30,
            fill=(238, 0, 0),
            stroke_ratio=1 / 15,
            stroke_fill=(255, 255, 153),
        )
    except ValueError:
        raise TextOverLength(text)
    return frame.save_png()


add_meme(
    "good_news",
    good_news,
    min_texts=1,
    max_texts=1,
    default_texts=["悲报"],
    keywords=["喜报"],
    date_created=datetime(2021, 12, 15),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from io import BytesIO
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"


def osu(images, texts: list[str], args) -> BytesIO:
    text = texts[0]
    frame = BuildImage.open(img_dir / "osu.png")
    try:
        frame.draw_text(
            (80, 80, 432, 432),
            text,
            max_fontsize=192,
            min_fontsize=80,
            weight="bold",
            fill="white",
            lines_align="center",
            fontname="Aller",
        )
    except ValueError:
        raise TextOverLength(text)

    return frame.save_png()


add_meme(
    "osu",
    osu,
    min_texts=1,
    max_texts=1,
    default_texts=["hso!"],
    keywords=["osu"],
    date_created=datetime(2023, 7, 27),
    date_modified=datetime(2023, 7, 27),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import CommandShortcut, add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"


def wakeup(images, texts: list[str], args):
    text = texts[0]
    frame = BuildImage.open(img_dir / "0.jpg")
    try:
        frame.draw_text((310, 270, 460, 380), text, max_fontsize=90, min_fontsize=50)
    except ValueError:
        raise TextOverLength(text)
    frame.draw_text(
        (50, 610, 670, 720), f"{text}起来了", max_fontsize=110, min_fontsize=70
    )
    return frame.save_jpg()


add_meme(
    "wakeup",
    wakeup,
    min_texts=1,
    max_texts=1,
    default_texts=["好"],
    keywords=["好起来了"],
    shortcuts=[
        CommandShortcut(
            key=r"(?P<text>\S{1,4})\s+起来了",
            args=["{text}"],
            humanized="xx 起来了",
        )
    ],
    date_created=datetime(2022, 6, 12),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"

default_text = "平安名すみれ"


def police(images: list[BuildImage], texts: list[str], args):
    text = default_text if not texts else texts[0]
    text_frame = BuildImage.new("RGBA", (250, 85))
    try:
        text_frame.draw_text(
            (0, 0, 250, 85),
            text,
            fontname="Noto Serif SC",
            max_fontsize=60,
            min_fontsize=20,
        )
    except ValueError:
        raise TextOverLength(text)
    img = images[0].convert("RGBA").square().resize((245, 245))

    frame = BuildImage.open(img_dir / "0.png")
    frame.paste(img, (224, 46), below=True)
    frame.paste(text_frame, (220, 395), alpha=True)
    return frame.save_jpg()


def police1(images: list[BuildImage], texts, args):
    img = (
        images[0]
        .convert("RGBA")
        .resize((60, 75), keep_ratio=True)
        .rotate(16, expand=True)
    )
    frame = BuildImage.open(img_dir / "1.png")
    frame.paste(img, (37, 291), below=True)
    return frame.save_jpg()


add_meme(
    "police",
    police,
    min_images=1,
    max_images=1,
    max_texts=1,
    min_texts=0,
    default_texts=[default_text],
    keywords=["出警"],
    date_created=datetime(2022, 2, 23),
    date_modified=datetime(2024, 9, 6),
)

add_meme(
    "police1",
    police1,
    min_images=1,
    max_images=1,
    keywords=["警察"],
    date_created=datetime(2022, 3, 12),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime

from PIL.Image import Transpose
from pil_utils import BuildImage

from meme_generator import add_meme

defalut_text = "如何提高社交质量 : \n远离以下头像的人"


def keep_away(images: list[BuildImage], texts: list[str], args):
    def trans(img: BuildImage, n: int) -> BuildImage:
        img = img.convert("RGBA").square().resize((100, 100))
        if n < 4:
            return img.rotate(n * 90)
        else:
            return img.transpose(Transpose.FLIP_LEFT_RIGHT).rotate((n - 4) * 90)

    def paste(img: BuildImage):
        nonlocal count
        y = 90 if count < 4 else 190
        frame.paste(img, ((count % 4) * 100, y))
        count += 1

    text = texts[0] if texts else defalut_text
    frame = BuildImage.new("RGB", (400, 290), "white")
    frame.draw_text((10, 10, 390, 80), text, max_fontsize=40, halign="left")
    count = 0
    num_per_user = 8 // len(images)
    for image in images:
        for n in range(num_per_user):
            paste(trans(image, n))
    num_left = 8 - num_per_user * len(images)
    for n in range(num_left):
        paste(trans(images[-1], n + num_per_user))

    return frame.save_jpg()


add_meme(
    "keep_away",
    keep_away,
    min_images=1,
    max_images=8,
    min_texts=0,
    max_texts=1,
    default_texts=[defalut_text],
    keywords=["远离"],
    date_created=datetime(2022, 5, 31),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags

img_dir = Path(__file__).parent / "images"


def captain(images: list[BuildImage], texts, args):
    if len(images) == 2:
        images.append(images[-1])

    bg0 = BuildImage.open(img_dir / "0.png")
    bg1 = BuildImage.open(img_dir / "1.png")
    bg2 = BuildImage.open(img_dir / "2.png")

    frame = BuildImage.new("RGBA", (640, 440 * len(images)), "white")
    for i in range(len(images)):
        bg = bg0 if i < len(images) - 2 else bg1 if i == len(images) - 2 else bg2
        images[i] = images[i].convert("RGBA").square().resize((250, 250))
        bg = bg.copy().paste(images[i], (350, 85))
        frame.paste(bg, (0, 440 * i))

    return frame.save_jpg()


add_meme(
    "captain",
    captain,
    min_images=2,
    max_images=5,
    keywords=["舰长"],
    tags=MemeTags.captain,
    date_created=datetime(2022, 10, 9),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from arclet.alconna import store_true
from PIL.Image import Image as IMG
from pil_utils import BuildImage
from pydantic import Field

from meme_generator import MemeArgsModel, MemeArgsType, ParserOption, add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"

help_text = "是否将图片变为圆形"


class Model(MemeArgsModel):
    circle: bool = Field(False, description=help_text)


args_type = MemeArgsType(
    args_model=Model,
    args_examples=[Model(circle=False), Model(circle=True)],
    parser_options=[
        ParserOption(
            names=["--circle", "圆"],
            default=False,
            action=store_true,
            help_text=help_text,
        ),
    ],
)


def dog_dislike(images: list[BuildImage], texts: list[str], args: Model):
    location = [
        (36, 408),
        (36, 410),
        (40, 375),
        (40, 355),
        (36, 325),
        (28, 305),
        (28, 305),
        (28, 305),
        (28, 305),
        (28, 285),
        (28, 285),
        (28, 285),
        (28, 285),
        (28, 290),
        (30, 295),
        (30, 300),
        (30, 300),
        (30, 300),
        (30, 300),
        (30, 300),
        (30, 300),
        (28, 298),
        (26, 296),
        (24, 294),
        (28, 294),
        (26, 294),
        (24, 294),
        (35, 294),
        (115, 330),
        (150, 355),
        (180, 420),
        (180, 450),
        (150, 450),
        (150, 450),
    ]
    head = images[0].convert("RGBA").resize((122, 122), keep_ratio=True)
    if args.circle:
        head = head.circle()
    frames: list[IMG] = []
    for i in range(34):
        frame = BuildImage.open(img_dir / f"{i}.png")
        frame.paste(head, location[i], alpha=True)
        frames.append(frame.image)
    return save_gif(frames, 0.08)


add_meme(
    "dog_dislike",
    dog_dislike,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=0,
    args_type=args_type,
    keywords=["狗都不玩"],
    date_created=datetime(2023, 11, 16),
    date_modified=datetime(2023, 11, 16),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage, Text2Image
from pydantic import Field

from meme_generator import (
    CommandShortcut,
    MemeArgsModel,
    MemeArgsType,
    ParserArg,
    ParserOption,
    add_meme,
)

img_dir = Path(__file__).parent / "images"

help_text = "指定名字"


class Model(MemeArgsModel):
    name: str = Field("", description=help_text)


args_type = MemeArgsType(
    args_model=Model,
    parser_options=[
        ParserOption(
            names=["-n", "--name"],
            args=[ParserArg(name="name", value="str")],
            help_text=help_text,
        ),
    ],
)


def steam_message(images: list[BuildImage], texts: list[str], args: Model):
    name = args.name or (args.user_infos[0].name if args.user_infos else "好友")
    game = texts[0]

    text_name = Text2Image.from_text(name, 65, fill="#e3ffc2")
    text_play = Text2Image.from_text("正在玩", 62, fill="#d1d1c0")
    text_game = Text2Image.from_text(game, 65, fill="#91c257")

    avatar_w = 280
    padding_h = 50
    padding_v = 80
    margin_rec = 6
    rec_w = 15
    margin_text = 80
    text_w = max(text_name.width, text_play.width, text_game.width)
    text_w = max(text_w, 1300)
    text_x = padding_h + avatar_w + margin_rec + rec_w + margin_text
    frame_w = text_x + text_w + padding_h
    frame_h = padding_v * 2 + avatar_w

    frame = BuildImage.new("RGBA", (frame_w, frame_h), "#14161f")
    frame.paste(
        BuildImage.new("RGB", (avatar_w, avatar_w), "#191b23"), (padding_h, padding_v)
    )
    avatar = images[0].convert("RGBA").resize((avatar_w, avatar_w), keep_ratio=True)
    frame.paste(avatar, (padding_h, padding_v), alpha=True)
    rec_x = padding_h + avatar_w + margin_rec
    frame.draw_rectangle(
        (rec_x, padding_v, rec_x + rec_w, frame_h - padding_v), fill="#6cbe48"
    )
    logo = BuildImage.open(img_dir / "logo.png")
    frame.alpha_composite(logo, (frame_w - 870, -370))
    text_play.draw_on_image(frame.image, (text_x, (frame_h - text_play.height) // 2))
    text_name.draw_on_image(
        frame.image, (text_x, padding_v + 40 - text_name.height // 2)
    )
    text_game.draw_on_image(
        frame.image, (text_x, frame_h - padding_v - 40 - text_game.height // 2)
    )

    return frame.save_jpg()


add_meme(
    "steam_message",
    steam_message,
    min_images=1,
    max_images=1,
    min_texts=1,
    max_texts=1,
    default_texts=["黑神话：悟空"],
    args_type=args_type,
    keywords=["steam消息"],
    shortcuts=[
        CommandShortcut(
            key=r"(?P<name>\S+)正在玩(?P<game>\S+)",
            args=["--name", "{name}", "{game}"],
            humanized="xx正在玩xx",
        )
    ],
    date_created=datetime(2024, 8, 21),
    date_modified=datetime(2024, 8, 21),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags
from meme_generator.utils import make_jpg_or_gif

img_dir = Path(__file__).parent / "images"


def maimai_awaken(images: list[BuildImage], texts, args):
    frame = BuildImage.open(img_dir / "0.png")

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = (
            imgs[0].convert("RGBA").square().resize((250, 250)).rotate(-25, expand=True)
        )
        return frame.copy().paste(img, (134, 134), alpha=True, below=True)

    return make_jpg_or_gif(images, make)


add_meme(
    "maimai_awaken",
    maimai_awaken,
    min_images=1,
    max_images=1,
    keywords=["旅行伙伴觉醒"],
    tags=MemeTags.maimai,
    date_created=datetime(2023, 7, 19),
    date_modified=datetime(2023, 7, 19),
)



from datetime import datetime
from pathlib import Path

from PIL import ImageEnhance
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"

defalut_text = "你刚才说的话不是很礼貌！"


def incivilization(images: list[BuildImage], texts: list[str], args):
    frame = BuildImage.open(img_dir / "0.png")
    points = ((0, 20), (154, 0), (164, 153), (22, 180))
    img = images[0].convert("RGBA").circle().resize((150, 150)).perspective(points)
    image = ImageEnhance.Brightness(img.image).enhance(0.8)
    frame.paste(image, (137, 151), alpha=True)
    text = texts[0] if texts else defalut_text
    try:
        frame.draw_text(
            (57, 42, 528, 117),
            text,
            weight="bold",
            max_fontsize=50,
            min_fontsize=20,
            allow_wrap=True,
        )
    except ValueError:
        raise TextOverLength(text)
    return frame.save_jpg()


add_meme(
    "incivilization",
    incivilization,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    default_texts=[defalut_text],
    keywords=["不文明"],
    date_created=datetime(2022, 10, 12),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def pound(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").square()
    # fmt: off
    locs = [
        (135, 240, 138, 47), (135, 240, 138, 47), (150, 190, 105, 95), (150, 190, 105, 95),
        (148, 188, 106, 98), (146, 196, 110, 88), (145, 223, 112, 61), (145, 223, 112, 61)
    ]
    # fmt: on
    frames: list[IMG] = []
    for i in range(8):
        frame = BuildImage.open(img_dir / f"{i}.png")
        x, y, w, h = locs[i]
        frame.paste(img.resize((w, h)), (x, y), below=True)
        frames.append(frame.image)
    return save_gif(frames, 0.05)


add_meme(
    "pound",
    pound,
    min_images=1,
    max_images=1,
    keywords=["捣"],
    date_created=datetime(2022, 3, 30),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import FrameAlignPolicy, Maker, make_gif_or_combined_gif


def windmill_turn(images: list[BuildImage], texts, args):
    def maker(i: int) -> Maker:
        def make(imgs: list[BuildImage]) -> BuildImage:
            img = imgs[0].convert("RGBA").resize((300, 300), keep_ratio=True)
            frame = BuildImage.new("RGBA", (600, 600), "white")
            frame.paste(img, alpha=True)
            frame.paste(img.rotate(90), (0, 300), alpha=True)
            frame.paste(img.rotate(180), (300, 300), alpha=True)
            frame.paste(img.rotate(270), (300, 0), alpha=True)
            return frame.rotate(i * 18).crop((50, 50, 550, 550))

        return make

    return make_gif_or_combined_gif(
        images, maker, 5, 0.05, FrameAlignPolicy.extend_loop
    )


add_meme(
    "windmill_turn",
    windmill_turn,
    min_images=1,
    max_images=1,
    keywords=["风车转"],
    date_created=datetime(2022, 12, 13),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def knock(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").square()
    # fmt: off
    locs = [(60, 308, 210, 195), (60, 308, 210, 198), (45, 330, 250, 172), (58, 320, 218, 180),
            (60, 310, 215, 193), (40, 320, 250, 285), (48, 308, 226, 192), (51, 301, 223, 200)]
    # fmt: on
    frames: list[IMG] = []
    for i in range(8):
        frame = BuildImage.open(img_dir / f"{i}.png")
        x, y, w, h = locs[i]
        frame.paste(img.resize((w, h)), (x, y), below=True)
        frames.append(frame.image)
    return save_gif(frames, 0.04)


add_meme(
    "knock",
    knock,
    min_images=1,
    max_images=1,
    keywords=["敲"],
    tags=MemeTags.gura,
    date_created=datetime(2022, 4, 14),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength
from meme_generator.utils import make_jpg_or_gif

img_dir = Path(__file__).parent / "images"

default_text = "朋友\n先看看这个图标再说话"


def look_this_icon(images: list[BuildImage], texts: list[str], args):
    text = texts[0] if texts else default_text
    frame = BuildImage.open(img_dir / "nmsl.png")
    try:
        frame.draw_text(
            (0, 933, 1170, 1143),
            text,
            lines_align="center",
            weight="bold",
            max_fontsize=100,
            min_fontsize=50,
        )
    except ValueError:
        raise TextOverLength(text)

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA").resize((515, 515), keep_ratio=True)
        return frame.copy().paste(img, (599, 403), below=True)

    return make_jpg_or_gif(images, make)


add_meme(
    "look_this_icon",
    look_this_icon,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    default_texts=[default_text],
    keywords=["看图标"],
    date_created=datetime(2022, 10, 7),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import MemeArgsModel, add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"


def coupon(images: list[BuildImage], texts: list[str], args: MemeArgsModel):
    img = images[0].convert("RGBA").circle().resize((60, 60))
    name = args.user_infos[0].name if args.user_infos else ""
    text = (texts[0] if texts else f"{name}陪睡券") + "\n（永久有效）"

    text_img = BuildImage.new("RGBA", (250, 100))
    try:
        text_img.draw_text(
            (0, 0, text_img.width, text_img.height),
            text,
            lines_align="center",
            max_fontsize=30,
            min_fontsize=15,
        )
    except ValueError:
        raise TextOverLength(text)

    frame = BuildImage.open(img_dir / "0.png")
    frame.paste(img.rotate(22, expand=True), (164, 85), alpha=True)
    frame.paste(text_img.rotate(22, expand=True), (94, 108), alpha=True)
    return frame.save_jpg()


add_meme(
    "coupon",
    coupon,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    keywords=["兑换券"],
    date_created=datetime(2022, 3, 12),
    date_modified=datetime(2023, 2, 14),
)



from pathlib import Path

from meme_generator.config import meme_config as config
from meme_generator.manager import add_meme as add_meme
from meme_generator.manager import get_meme as get_meme
from meme_generator.manager import get_meme_keys as get_meme_keys
from meme_generator.manager import get_memes as get_memes
from meme_generator.manager import load_meme as load_meme
from meme_generator.manager import load_memes as load_memes
from meme_generator.meme import CommandShortcut as CommandShortcut
from meme_generator.meme import Meme as Meme
from meme_generator.meme import MemeArgsModel as MemeArgsModel
from meme_generator.meme import MemeArgsType as MemeArgsType
from meme_generator.meme import MemeParamsType as MemeParamsType
from meme_generator.meme import ParserArg as ParserArg
from meme_generator.meme import ParserOption as ParserOption
from meme_generator.version import __version__ as __version__

if config.meme.load_builtin_memes:
    for path in (Path(__file__).parent / "memes").iterdir():
        load_meme(f"meme_generator.memes.{path.name}")
for meme_dir in config.meme.meme_dirs:
    load_memes(meme_dir)



import math
import random
from datetime import datetime

from PIL.Image import Image as IMG
from pil_utils import BuildImage, Text2Image

from meme_generator import add_meme
from meme_generator.utils import save_gif


def douyin(images, texts: list[str], args):
    text = texts[0]
    text = " ".join(text.splitlines())
    fontsize = 200
    offset = round(fontsize * 0.05)
    px = 70
    py = 30
    bg_color = "#1C0B1B"
    frame = Text2Image.from_text(
        text, fontsize, fill="#FF0050", stroke_fill="#FF0050", stroke_width=5
    ).to_image(bg_color=bg_color, padding=(px + offset * 2, py + offset * 2, px, py))
    Text2Image.from_text(
        text, fontsize, fill="#00F5EB", stroke_fill="#00F5EB", stroke_width=5
    ).draw_on_image(frame, (px, py))
    Text2Image.from_text(
        text, fontsize, fill="white", stroke_fill="white", stroke_width=5
    ).draw_on_image(frame, (px + offset, py + offset))
    frame = BuildImage(frame)

    width = frame.width - px
    height = frame.height - py
    frame_num = 10
    devide_num = 6
    seed = 20 * 0.05
    frames: list[IMG] = []
    for _ in range(frame_num):
        new_frame = frame.copy()
        h_seeds = [
            math.fabs(math.sin(random.random() * devide_num)) for _ in range(devide_num)
        ]
        h_seed_sum = sum(h_seeds)
        h_seeds = [s / h_seed_sum for s in h_seeds]
        direction = 1
        last_yn = 0
        last_h = 0
        for i in range(devide_num):
            yn = last_yn + last_h
            h = max(round(height * h_seeds[i]), 2)
            last_yn = yn
            last_h = h
            direction = -direction
            piece = new_frame.copy().crop((px, yn, px + width, yn + h))
            new_frame.paste(piece, (px + round(i * direction * seed), yn))
        # 透视变换
        move_x = 64
        points = (
            (move_x, 0),
            (new_frame.width + move_x, 0),
            (new_frame.width, new_frame.height),
            (0, new_frame.height),
        )
        new_frame = new_frame.perspective(points)
        bg = BuildImage.new("RGBA", new_frame.size, bg_color)
        bg.paste(new_frame, alpha=True)
        frames.append(bg.image)

    return save_gif(frames, 0.2)


add_meme(
    "douyin",
    douyin,
    min_texts=1,
    max_texts=1,
    default_texts=["douyin"],
    keywords=["douyin"],
    date_created=datetime(2022, 10, 29),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import make_jpg_or_gif

img_dir = Path(__file__).parent / "images"


def add_chaos(images: list[BuildImage], texts, args):
    banner = BuildImage.open(img_dir / "0.png")

    def make(imgs: list[BuildImage]) -> BuildImage:
        return imgs[0].convert("RGBA").resize_width(240).paste(banner)

    return make_jpg_or_gif(images, make)


add_meme(
    "add_chaos",
    add_chaos,
    min_images=1,
    max_images=1,
    keywords=["添乱", "给社会添乱"],
    date_created=datetime(2023, 6, 21),
    date_modified=datetime(2023, 6, 21),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import MemeArgsModel, add_meme
from meme_generator.exception import TextOverLength
from meme_generator.utils import make_jpg_or_gif

img_dir = Path(__file__).parent / "images"

default_text = "一起玩{name}吧！"


def together(images: list[BuildImage], texts: list[str], args: MemeArgsModel):
    frame = BuildImage.open(img_dir / "0.png")
    name = args.user_infos[0].name if args.user_infos else ""
    text = texts[0] if texts else default_text.format(name=name)
    try:
        frame.draw_text(
            (10, 140, 190, 190),
            text,
            weight="bold",
            max_fontsize=50,
            min_fontsize=10,
            allow_wrap=True,
        )
    except ValueError:
        raise TextOverLength(text)

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA").resize((63, 63), keep_ratio=True)
        return frame.copy().paste(img, (132, 36), alpha=True)

    return make_jpg_or_gif(images, make)


add_meme(
    "together",
    together,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    default_texts=[default_text],
    keywords=["一起"],
    date_created=datetime(2022, 10, 13),
    date_modified=datetime(2023, 3, 29),
)



from datetime import datetime
from io import BytesIO
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import make_jpg_or_gif

img_dir = Path(__file__).parent / "images"


def raise_image(images: list[BuildImage], texts, args) -> BytesIO:
    inner_size = (599, 386)
    paste_pos = (134, 91)

    bg = BuildImage.open(img_dir / "raise_image.png")

    def make_frame(imgs: list[BuildImage]) -> BuildImage:
        inner_frame = BuildImage.new("RGBA", inner_size, "white")
        inner_frame = inner_frame.paste(
            imgs[0].convert("RGBA").resize(inner_size, keep_ratio=True),
            alpha=True,
        )
        return bg.copy().paste(inner_frame, paste_pos, alpha=True, below=True)

    return make_jpg_or_gif(images, make_frame)


add_meme(
    "raise_image",
    raise_image,
    min_images=1,
    max_images=1,
    keywords=["举"],
    date_created=datetime(2023, 8, 9),
    date_modified=datetime(2023, 8, 9),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags
from meme_generator.utils import FrameAlignPolicy, Maker, make_gif_or_combined_gif

img_dir = Path(__file__).parent / "images"


def shiroko_pero(images: list[BuildImage], texts, args):
    mask = BuildImage.open(img_dir / "mask.png").convert("RGBA")

    def maker(i) -> Maker:
        def make(imgs: list[BuildImage]) -> BuildImage:
            suika = imgs[0].convert("RGBA").resize((245, 245), keep_ratio=True)
            frame = BuildImage.open(img_dir / f"{i}.png").convert("RGBA")
            suika_mask = BuildImage.new("RGBA", (245, 245), (0, 0, 0, 0))
            suika_mask.image.paste(suika.image, (0, 0), mask.image)
            frame.paste(suika_mask, (105, 178), below=True)
            return frame

        return make

    return make_gif_or_combined_gif(
        images, maker, 4, 0.06, FrameAlignPolicy.extend_loop
    )


add_meme(
    "shiroko_pero",
    shiroko_pero,
    min_images=1,
    max_images=1,
    keywords=["白子舔"],
    tags=MemeTags.shiroko,
    date_created=datetime(2024, 8, 10),
    date_modified=datetime(2024, 8, 10),
)



import random
from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from PIL.Image import Palette
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import make_jpg_or_gif

img_dir = Path(__file__).parent / "images"


def get_dominant_colors(img: IMG) -> list[tuple[int, int, int]]:
    img = img.convert("P", palette=Palette.ADAPTIVE, colors=20)
    palette = img.getpalette()
    assert palette
    color_indexs = sorted(img.getcolors(), reverse=True)  # type: ignore
    colors = [tuple(palette[i * 3 : i * 3 + 3]) for _, i in color_indexs]
    colors = list(
        filter(lambda c: c[0] * 0.299 + c[1] * 0.578 + c[2] * 0.114 < 200, colors)
    )
    return colors  # type: ignore


def dont_touch(images: list[BuildImage], texts, args):
    frame = BuildImage.open(img_dir / "0.png")
    mask = BuildImage.open(img_dir / "mask.png").convert("L")

    def paste_random_blocks(img: BuildImage, colors: list[tuple[int, int, int]]):
        x1, y1, x2, y2 = 200, 300, 400, 650
        block_locs = []
        for _ in range(150):
            x = random.randint(x1, x2)
            y = random.randint(y1, y2)
            if mask.image.getpixel((x, y)) == 0:
                continue
            if any(abs(x - x_) < 13 and abs(y - y_) < 13 for x_, y_ in block_locs):
                continue
            block_locs.append((x, y))
            color = random.choice(colors)
            block = BuildImage.new("RGBA", (10, 10), color)
            block = block.rotate(45, expand=True)
            img.paste(block, (x, y), alpha=True)

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0]
        img_frame = frame.copy()
        colors = get_dominant_colors(img.image)
        paste_random_blocks(img_frame, colors)
        img = img.convert("RGBA").resize((250, 250), keep_ratio=True, inside=True)
        return img_frame.paste(img, (25, 460), alpha=True)

    return make_jpg_or_gif(images, make)


add_meme(
    "dont_touch",
    dont_touch,
    min_images=1,
    max_images=1,
    keywords=["别碰"],
    date_created=datetime(2023, 4, 27),
    date_modified=datetime(2023, 4, 27),
)



from datetime import datetime

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif


def funny_mirror(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").square().resize((500, 500))
    frames: list[IMG] = [img.image]
    coeffs = [0.01, 0.03, 0.05, 0.08, 0.12, 0.17, 0.23, 0.3, 0.4, 0.6]
    borders = [25, 52, 67, 83, 97, 108, 118, 128, 138, 148]
    for i in range(10):
        new_size = 500 - borders[i] * 2
        new_img = img.distort((coeffs[i], 0, 0, 0)).resize_canvas((new_size, new_size))
        frames.append(new_img.resize((500, 500)).image)
    frames.extend(frames[::-1])
    return save_gif(frames, 0.05)


add_meme(
    "funny_mirror",
    funny_mirror,
    min_images=1,
    max_images=1,
    keywords=["哈哈镜"],
    date_created=datetime(2022, 3, 13),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def hug(images: list[BuildImage], texts, args):
    self_head = images[0].convert("RGBA").circle().resize((120, 120))
    user_head = images[1].convert("RGBA").circle().resize((105, 105))
    # fmt: off
    user_locs = [
        (108, 15), (107, 14), (104, 16), (102, 14), (104, 15),
        (108, 15), (108, 15), (103, 16), (102, 15), (104, 14)
    ]
    self_locs = [
        (78, 120), (115, 130), (0, 0), (110, 100), (80, 100),
        (75, 115), (105, 127), (0, 0), (110, 98), (80, 105)
    ]
    rotate_num = [-48, -18, 0, 38, 31, -43, -22, 0, 34, 35]
    # fmt: on
    frames: list[IMG] = []
    for i in range(10):
        frame = BuildImage.open(img_dir / f"{i}.png")
        frame.paste(user_head, user_locs[i], below=True)
        img = self_head.rotate(rotate_num[i], expand=True)
        frame.paste(img, self_locs[i], below=True)
        frames.append(frame.image)
    return save_gif(frames, 0.05)


add_meme(
    "hug",
    hug,
    min_images=2,
    max_images=2,
    keywords=["抱", "抱抱"],
    date_created=datetime(2024, 8, 6),
    date_modified=datetime(2024, 8, 6),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def capoo_rub(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").square().resize((180, 180))
    frames: list[IMG] = []
    locs = [
        (178, 184, 78, 260),
        (178, 174, 84, 269),
        (178, 174, 84, 269),
        (178, 178, 84, 264),
    ]
    for i in range(4):
        frame = BuildImage.open(img_dir / f"{i}.png")
        w, h, x, y = locs[i]
        frame.paste(img.resize((w, h)), (x, y), below=True)
        frames.append(frame.image)
    return save_gif(frames, 0.1)


add_meme(
    "capoo_rub",
    capoo_rub,
    min_images=1,
    max_images=1,
    keywords=["咖波蹭", "咖波贴"],
    tags=MemeTags.capoo,
    date_created=datetime(2022, 11, 29),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def rise_dead(images: list[BuildImage], texts, args):
    locs = [
        ((81, 55), ((0, 2), (101, 0), (103, 105), (1, 105))),
        ((74, 49), ((0, 3), (104, 0), (106, 108), (1, 108))),
        ((-66, 36), ((0, 0), (182, 5), (184, 194), (1, 185))),
        ((-231, 55), ((0, 0), (259, 4), (276, 281), (13, 278))),
    ]
    img = images[0].convert("RGBA").square().resize((150, 150))
    imgs = [img.perspective(points) for _, points in locs]
    frames: list[IMG] = []
    for i in range(34):
        frame = BuildImage.open(img_dir / f"{i}.png")
        if i <= 28:
            idx = 0 if i <= 25 else i - 25
            x, y = locs[idx][0]
            if i % 2 == 1:
                x += 1
                y -= 1
            frame.paste(imgs[idx], (x, y), below=True)
        frames.append(frame.image)
    return save_gif(frames, 0.15)


add_meme(
    "rise_dead",
    rise_dead,
    min_images=1,
    max_images=1,
    keywords=["诈尸", "秽土转生"],
    date_created=datetime(2022, 11, 8),
    date_modified=datetime(2023, 2, 14),
)



"""本模块为 Pydantic 版本兼容层模块

为兼容 Pydantic V1 与 V2 版本，定义了一系列兼容函数与类供使用。
"""

from typing import Any, Optional, TypeVar, Union

from pydantic import VERSION, BaseModel

T = TypeVar("T")

PYDANTIC_V2 = int(VERSION.split(".", 1)[0]) == 2


__all__ = (
    "model_dump",
    "model_json_schema",
    "type_validate_python",
    "type_validate_json",
)


if PYDANTIC_V2:  # pragma: pydantic-v2
    from pydantic import TypeAdapter as TypeAdapter
    from pydantic.json_schema import DEFAULT_REF_TEMPLATE

    def model_dump(
        model: BaseModel,
        include: Optional[set[str]] = None,
        exclude: Optional[set[str]] = None,
        by_alias: bool = False,
        exclude_unset: bool = False,
        exclude_defaults: bool = False,
        exclude_none: bool = False,
    ) -> dict[str, Any]:
        return model.model_dump(
            include=include,
            exclude=exclude,
            by_alias=by_alias,
            exclude_unset=exclude_unset,
            exclude_defaults=exclude_defaults,
            exclude_none=exclude_none,
        )

    def model_json_schema(
        model: type[BaseModel],
        by_alias: bool = True,
        ref_template: str = DEFAULT_REF_TEMPLATE,
    ) -> dict[str, Any]:
        return model.model_json_schema(by_alias=by_alias, ref_template=ref_template)

    def type_validate_python(type_: type[T], data: Any) -> T:
        """Validate data with given type."""
        return TypeAdapter(type_).validate_python(data)

    def type_validate_json(type_: type[T], data: Union[str, bytes]) -> T:
        """Validate JSON with given type."""
        return TypeAdapter(type_).validate_json(data)


else:  # pragma: pydantic-v1
    from pydantic import parse_obj_as, parse_raw_as
    from pydantic.schema import default_ref_template

    def model_dump(
        model: BaseModel,
        include: Optional[set[str]] = None,
        exclude: Optional[set[str]] = None,
        by_alias: bool = False,
        exclude_unset: bool = False,
        exclude_defaults: bool = False,
        exclude_none: bool = False,
    ) -> dict[str, Any]:
        return model.dict(
            include=include,
            exclude=exclude,
            by_alias=by_alias,
            exclude_unset=exclude_unset,
            exclude_defaults=exclude_defaults,
            exclude_none=exclude_none,
        )

    def model_json_schema(
        model: type[BaseModel],
        by_alias: bool = True,
        ref_template: str = default_ref_template,
    ) -> dict[str, Any]:
        return model.schema(by_alias=by_alias, ref_template=ref_template)

    def type_validate_python(type_: type[T], data: Any) -> T:
        """Validate data with given type."""
        return parse_obj_as(type_, data)

    def type_validate_json(type_: type[T], data: Union[str, bytes]) -> T:
        """Validate JSON with given type."""
        return parse_raw_as(type_, data)



from datetime import datetime
from pathlib import Path

import dateparser
from pil_utils import BuildImage
from pydantic import Field

from meme_generator import (
    MemeArgsModel,
    MemeArgsType,
    ParserArg,
    ParserOption,
    add_meme,
)
from meme_generator.exception import TextOverLength


class Model(MemeArgsModel):
    time: str = Field("", description="指定时间")


args_type = MemeArgsType(
    args_model=Model,
    parser_options=[
        ParserOption(
            names=["-t", "--time"],
            args=[ParserArg(name="time", value="str")],
            help_text="指定时间",
        ),
    ],
)


img_dir = Path(__file__).parent / "images"


def certificate(images, texts: list[str], args: Model):
    time = datetime.now()
    if args.time and (parsed_time := dateparser.parse(args.time)):
        time = parsed_time

    frame = BuildImage.open(img_dir / "0.png")

    try:
        frame.draw_text(
            (340, 660, 770, 800),
            texts[0],
            allow_wrap=False,
            max_fontsize=80,
            min_fontsize=20,
        )
    except ValueError:
        raise TextOverLength(texts[0])
    try:
        frame.draw_text(
            (565, 1040, 2100, 1320),
            texts[1],
            fill="red",
            allow_wrap=True,
            max_fontsize=120,
            min_fontsize=60,
        )
    except ValueError:
        raise TextOverLength(texts[1])
    try:
        frame.draw_text(
            (1500, 1400, 2020, 1520),
            texts[2],
            allow_wrap=False,
            max_fontsize=60,
            min_fontsize=20,
        )
    except ValueError:
        raise TextOverLength(texts[2])
    try:
        frame.draw_text(
            (450, 850, 2270, 1080),
            texts[3]
            if len(texts) >= 4
            else "　　在本学年第一学期中表现优秀，被我校决定评为",
            allow_wrap=True,
            max_fontsize=80,
            min_fontsize=40,
            halign="left",
            valign="top",
        )
    except ValueError:
        raise TextOverLength(texts[3])

    frame.draw_text(
        (1565, 1525, 1700, 1600),
        f"{time.year:04d}",
        allow_wrap=False,
        max_fontsize=60,
        min_fontsize=40,
    )
    frame.draw_text(
        (1752, 1525, 1816, 1600),
        f"{time.month:02d}",
        allow_wrap=False,
        max_fontsize=60,
        min_fontsize=40,
    )
    frame.draw_text(
        (1865, 1525, 1930, 1600),
        f"{time.day:02d}",
        allow_wrap=False,
        max_fontsize=60,
        min_fontsize=40,
    )

    return frame.save_png()


add_meme(
    "certificate",
    certificate,
    min_texts=3,
    max_texts=4,
    default_texts=["小王", "优秀学生", "一年一班"],
    args_type=args_type,
    keywords=["奖状", "证书"],
    date_created=datetime(2023, 12, 3),
    date_modified=datetime(2023, 12, 3),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"


def slogan(images, texts: list[str], args):
    frame = BuildImage.open(img_dir / "0.jpg")

    def draw(pos: tuple[float, float, float, float], text: str):
        try:
            frame.draw_text(
                pos, text, max_fontsize=40, min_fontsize=15, allow_wrap=True
            )
        except ValueError:
            raise TextOverLength(text)

    draw((10, 0, 294, 50), texts[0])
    draw((316, 0, 602, 50), texts[1])
    draw((10, 230, 294, 280), texts[2])
    draw((316, 230, 602, 280), texts[3])
    draw((10, 455, 294, 505), texts[4])
    draw((316, 455, 602, 505), texts[5])

    return frame.save_jpg()


add_meme(
    "slogan",
    slogan,
    min_texts=6,
    max_texts=6,
    default_texts=[
        "我们是谁？",
        "浙大人！",
        "到浙大来做什么？",
        "混！",
        "将来毕业后要做什么样的人？",
        "混混！",
    ],
    keywords=["口号"],
    date_created=datetime(2022, 6, 12),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags

img_dir = Path(__file__).parent / "images"


def karyl_point(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").rotate(7.5, expand=True).resize((225, 225))
    frame = BuildImage.open(img_dir / "0.png")
    frame.paste(img, (87, 790), alpha=True)
    return frame.save_png()


add_meme(
    "karyl_point",
    karyl_point,
    min_images=1,
    max_images=1,
    keywords=["凯露指"],
    tags=MemeTags.karyl,
    date_created=datetime(2022, 11, 16),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def suck(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").square()
    # fmt: off
    locs = [(82, 100, 130, 119), (82, 94, 126, 125), (82, 120, 128, 99), (81, 164, 132, 55),
            (79, 163, 132, 55), (82, 140, 127, 79), (83, 152, 125, 67), (75, 157, 140, 62),
            (72, 165, 144, 54), (80, 132, 128, 87), (81, 127, 127, 92), (79, 111, 132, 108)]
    # fmt: on
    frames: list[IMG] = []
    for i in range(12):
        bg = BuildImage.open(img_dir / f"{i}.png")
        frame = BuildImage.new("RGBA", bg.size, "white")
        x, y, w, h = locs[i]
        frame.paste(img.resize((w, h)), (x, y), alpha=True).paste(bg, alpha=True)
        frames.append(frame.image)
    return save_gif(frames, 0.08)


add_meme(
    "suck",
    suck,
    min_images=1,
    max_images=1,
    keywords=["吸", "嗦"],
    date_created=datetime(2022, 4, 20),
    date_modified=datetime(2023, 2, 14),
)
